import csv
from pathlib import Path
from typing import Dict, List, Tuple

import datasets
from datasets.download.download_manager import DownloadManager

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = r"""
@inproceedings{kabra-etal-2023-multi,
    title = "Multi-lingual and Multi-cultural Figurative Language Understanding",
    author = "Kabra, Anubha  and
      Liu, Emmy  and
      Khanuja, Simran  and
      Aji, Alham Fikri  and
      Winata, Genta  and
      Cahyawijaya, Samuel  and
      Aremu, Anuoluwapo  and
      Ogayo, Perez  and
      Neubig, Graham",
    editor = "Rogers, Anna  and
      Boyd-Graber, Jordan  and
      Okazaki, Naoaki",
    booktitle = "Findings of the Association for Computational Linguistics: ACL 2023",
    month = jul,
    year = "2023",
    address = "Toronto, Canada",
    publisher = "Association for Computational Linguistics",
    url = "https://aclanthology.org/2023.findings-acl.525",
    doi = "10.18653/v1/2023.findings-acl.525",
    pages = "8269--8284",
}
"""

_LOCAL = False
_LANGUAGES = ["ind", "jav", "sun"]
_DATASETNAME = "mabl"
_DESCRIPTION = r"""\
The MABL (Metaphors Across Borders and Languages) dataset is a collection of
6,366 figurative language expressions from seven languages, crafted to improve
multilingual models' understanding of figurative speech and its linguistic
variations. It was built by crowdsourcing native speakers to generate paired
metaphors that began with the same words but had different meanings, as well as
the literal interpretations of both phrases. Each expression was checked by
fluent speakers to ensure they were clear, appropriate, and followed the format,
discarding any that didn't meet these standards.
"""

_HOMEPAGE = "https://github.com/simran-khanuja/Multilingual-Fig-QA"
_LICENSE = Licenses.MIT.value
_URL = "https://raw.githubusercontent.com/simran-khanuja/Multilingual-Fig-QA/main/langdata/"

_SUPPORTED_TASKS = [Tasks.QUESTION_ANSWERING]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


def iso3to2(lang: str) -> str:
    """Convert 3-letter ISO code to its 2-letter equivalent"""
    iso_map = {"ind": "id", "jav": "jv", "sun": "su"}
    return iso_map[lang]


class MABLDataset(datasets.GeneratorBasedBuilder):
    """MABL dataset by Liu et al (2023)"""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    SEACROWD_SCHEMA_NAME = "qa"

    dataset_names = sorted([f"{_DATASETNAME}_{lang}" for lang in _LANGUAGES])
    BUILDER_CONFIGS = []
    for name in dataset_names:
        source_config = SEACrowdConfig(
            name=f"{name}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=name,
        )
        BUILDER_CONFIGS.append(source_config)
        seacrowd_config = SEACrowdConfig(
            name=f"{name}_seacrowd_{SEACROWD_SCHEMA_NAME}",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema=f"seacrowd_{SEACROWD_SCHEMA_NAME}",
            subset_id=name,
        )
        BUILDER_CONFIGS.append(seacrowd_config)

    # Add configuration that allows loading all languages at once.
    BUILDER_CONFIGS.extend(
        [
            # mabl_source
            SEACrowdConfig(
                name=f"{_DATASETNAME}_source",
                version=SOURCE_VERSION,
                description=f"{_DATASETNAME} source schema (all)",
                schema="source",
                subset_id=_DATASETNAME,
            ),
            # mabl_seacrowd_qa
            SEACrowdConfig(
                name=f"{_DATASETNAME}_seacrowd_{SEACROWD_SCHEMA_NAME}",
                version=SEACROWD_VERSION,
                description=f"{_DATASETNAME} SEACrowd schema (all)",
                schema=f"seacrowd_{SEACROWD_SCHEMA_NAME}",
                subset_id=_DATASETNAME,
            ),
        ]
    )

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "startphrase": datasets.Value("string"),
                    "ending1": datasets.Value("string"),
                    "ending2": datasets.Value("string"),
                    "labels": datasets.Value("string"),
                }
            )
        elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            features = schemas.qa_features
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: DownloadManager) -> List[datasets.SplitGenerator]:
        """Return SplitGenerators."""
        mabl_source_data = []
        languages = []

        lang = self.config.name.split("_")[1]
        if lang in _LANGUAGES:
            # Load data per language
            mabl_source_data.append(dl_manager.download_and_extract(_URL + f"{iso3to2(lang)}.csv"))
            languages.append(lang)
        else:
            # Load examples for all languages at once.
            # We run this block when mabl_source / mabl_seacrowd_qa was chosen.
            for lang in _LANGUAGES:
                mabl_source_data.append(dl_manager.download_and_extract(_URL + f"{iso3to2(lang)}.csv"))
                languages.append(lang)

        return [
            datasets.SplitGenerator(
                # The MABL paper mentions that due to the size of each subset,
                # they consider each split as a test set.
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepaths": mabl_source_data,
                    "split": "test",
                    "languages": languages,
                },
            )
        ]

    def _generate_examples(self, filepaths: List[Path], split: str, languages: List[str]) -> Tuple[int, Dict]:
        """Yield examples as (key, example) tuples"""

        startphrases = []
        endings1 = []
        endings2 = []
        labels = []

        for lang, filepath in zip(languages, filepaths):
            with open(filepath, encoding="utf-8") as f:
                csv_reader = csv.reader(f, delimiter=",")
                next(csv_reader, None)  # skip the headers
                for row in csv_reader:
                    # Unfortunately, the columns in the subfiles of the MABL
                    # dataset are inconsistent. For 'ind', it is [ending1,
                    # ending2, labels, startphrase].  But for 'jav' and 'sun',
                    # the labels and startphrase columns were switched.  Here,
                    # I'm just hard-coding the column names
                    if lang == "ind":
                        end1, end2, label, start = row
                    if lang == "jav" or lang == "sun":
                        end1, end2, start, label = row

                    startphrases.append(start)
                    endings1.append(end1)
                    endings2.append(end2)
                    labels.append(label)

        for idx, (start, end1, end2, label) in enumerate(zip(startphrases, endings1, endings2, labels)):
            if self.config.schema == "source":
                example = {
                    "id": str(idx),
                    "startphrase": start,
                    "ending1": end1,
                    "ending2": end2,
                    "labels": label,
                }
            elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
                # Create QA-specific items
                choices = [end1, end2]
                answer = choices[int(label)]

                # MABL doesn't differentiate between question and context.
                # It only contains a startphrase. Given that, I put the
                # startphrase in question and kept the context blank.
                example = {
                    "id": str(idx),
                    "question_id": idx,
                    "document_id": idx,
                    "question": start,
                    "type": "multiple_choice",
                    "choices": choices,
                    "context": "",
                    "answer": [answer],
                    "meta": {},
                }

            yield idx, example



from pathlib import Path
from typing import Dict, List, Tuple

import datasets
import pandas as pd
from datasets.download.download_manager import DownloadManager

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """
@article{SUTOYO2022108554,
title = {PRDECT-ID: Indonesian product reviews dataset for emotions classification tasks},
journal = {Data in Brief},
volume = {44},
pages = {108554},
year = {2022},
issn = {2352-3409},
doi = {https://doi.org/10.1016/j.dib.2022.108554},
url = {https://www.sciencedirect.com/science/article/pii/S2352340922007612},
author = {Rhio Sutoyo and Said Achmad and Andry Chowanda and Esther Widhi Andangsari and Sani M. Isa},
keywords = {Natural language processing, Text processing, Text mining, Emotions classification, Sentiment analysis},
abstract = {Recognizing emotions is vital in communication. Emotions convey
additional meanings to the communication process. Nowadays, people can
communicate their emotions on many platforms; one is the product review. Product
reviews in the online platform are an important element that affects customers’
buying decisions. Hence, it is essential to recognize emotions from the product
reviews. Emotions recognition from the product reviews can be done automatically
using a machine or deep learning algorithm. Dataset can be considered as the
fuel to model the recognizer. However, only a limited dataset exists in
recognizing emotions from the product reviews, particularly in a local language.
This research contributes to the dataset collection of 5400 product reviews in
Indonesian. It was carefully curated from various (29) product categories,
annotated with five emotions, and verified by an expert in clinical psychology.
The dataset supports an innovative process to build automatic emotion
classification on product reviews.}
}
"""

_LOCAL = False
_LANGUAGES = ["ind"]
_DATASETNAME = "prdect_id"
_DESCRIPTION = """
PRDECT-ID Dataset is a collection of Indonesian product review data annotated
with emotion and sentiment labels. The data were collected from one of the giant
e-commerce in Indonesia named Tokopedia. The dataset contains product reviews
from 29 product categories on Tokopedia that use the Indonesian language. Each
product review is annotated with a single emotion, i.e., love, happiness, anger,
fear, or sadness. The group of annotators does the annotation process to provide
emotion labels by following the emotions annotation criteria created by an
expert in clinical psychology. Other attributes related to the product review
are also extracted, such as Location, Price, Overall Rating, Number Sold, Total
Review, and Customer Rating, to support further research.
"""

_HOMEPAGE = "https://data.mendeley.com/datasets/574v66hf2v/1"
_LICENSE = Licenses.CC_BY_4_0.value
_URL = "https://data.mendeley.com/public-files/datasets/574v66hf2v/files/f258d159-c678-42f1-9634-edf091a0b1f3/file_downloaded"

_SUPPORTED_TASKS = [Tasks.SENTIMENT_ANALYSIS, Tasks.EMOTION_CLASSIFICATION]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class PrdectIDDataset(datasets.GeneratorBasedBuilder):
    """PRDECT-ID Dataset"""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    SEACROWD_SCHEMA_NAME = "text"

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_emotion_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}_emotion",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_sentiment_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}_sentiment",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_emotion_seacrowd_{SEACROWD_SCHEMA_NAME}",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema for emotion classification",
            schema=f"seacrowd_{SEACROWD_SCHEMA_NAME}",
            subset_id=f"{_DATASETNAME}_emotion",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_sentiment_seacrowd_{SEACROWD_SCHEMA_NAME}",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema for sentiment analysis",
            schema=f"seacrowd_{SEACROWD_SCHEMA_NAME}",
            subset_id=f"{_DATASETNAME}_sentiment",
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"
    CLASS_LABELS_EMOTION = ["Happy", "Sadness", "Anger", "Love", "Fear"]
    CLASS_LABELS_SENTIMENT = ["Positive", "Negative"]

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "Category": datasets.Value("string"),
                    "Product Name": datasets.Value("string"),
                    "Location": datasets.Value("string"),
                    "Price": datasets.Value("int32"),
                    "Overall Rating": datasets.Value("float32"),
                    "Number Sold": datasets.Value("int32"),
                    "Total Review": datasets.Value("int32"),
                    "Customer Rating": datasets.Value("int32"),
                    "Customer Review": datasets.Value("string"),
                    "Sentiment": datasets.ClassLabel(names=self.CLASS_LABELS_SENTIMENT),
                    "Emotion": datasets.ClassLabel(names=self.CLASS_LABELS_EMOTION),
                }
            )
        elif self.config.schema == "seacrowd_text":
            if self.config.subset_id == f"{_DATASETNAME}_emotion":
                features = schemas.text_features(label_names=self.CLASS_LABELS_EMOTION)
            elif self.config.subset_id == f"{_DATASETNAME}_sentiment":
                features = schemas.text_features(label_names=self.CLASS_LABELS_SENTIMENT)
            else:
                raise ValueError(f"Invalid subset: {self.config.subset_id}")
        else:
            raise ValueError(f"Schema '{self.config.schema}' is not defined.")

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        data_file = Path(dl_manager.download(_URL))
        return [datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs={"filepath": data_file})]

    def _generate_examples(self, filepath: Path) -> Tuple[int, Dict]:
        """Yield examples as (key, example) tuples"""
        df = pd.read_csv(filepath, encoding="utf-8")
        for idx, row in df.iterrows():
            if self.config.schema == "source":
                yield idx, dict(row)
            elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
                if self.config.subset_id == f"{_DATASETNAME}_emotion":
                    yield idx, {"id": idx, "text": row["Customer Review"], "label": row["Emotion"]}
                elif self.config.subset_id == f"{_DATASETNAME}_sentiment":
                    yield idx, {"id": idx, "text": row["Customer Review"], "label": row["Sentiment"]}
                else:
                    raise ValueError(f"Invalid subset: {self.config.subset_id}")



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from pathlib import Path
from typing import Dict, List, Tuple

import datasets
import pandas as pd

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@article{Tuhenay2021,
  title = {Perbandingan Klasifikasi Bahasa Menggunakan Metode Naïve Bayes Classifier (NBC) Dan Support Vector Machine (SVM)},
  volume = {4},
  ISSN = {2656-1948},
  url = {http://dx.doi.org/10.33387/jiko.v4i2.2958},
  DOI = {10.33387/jiko.v4i2.2958},
  number = {2},
  journal = {JIKO (Jurnal Informatika dan Komputer)},
  publisher = {LPPM Universitas Khairun},
  author = {Tuhenay,  Deglorians},
  year = {2021},
  month = aug,
  pages = {105-111}
}
"""

_DATASETNAME = "identifikasi_bahasa"

_DESCRIPTION = """\
The identifikasi-bahasa dataset includes text samples in Indonesian, Ambonese, and Javanese. \
Each entry is comprised of cleantext, representing the sentence content, and a label identifying the language. \
The manual input process involved grouping the data by language categories, \
with labels for language identification and cleantext representing sentence content. The dataset, excluding punctuation and numbers, \
consists of a minimum of 3,000 Ambonese, 10,000 Javanese, \
and 3,500 Indonesian language entries, meeting the research's minimum standard for effective language identification.
"""

_HOMEPAGE = "https://github.com/joanitolopo/identifikasi-bahasa"
_LANGUAGES = ["ind", "jav", "abs"]

_LICENSE = Licenses.APACHE_2_0.value
_LOCAL = False

_URLS = {
    _DATASETNAME: "https://github.com/joanitolopo/identifikasi-bahasa/raw/main/DataKlasifikasi.xlsx",
}

_SUPPORTED_TASKS = [Tasks.LANGUAGE_IDENTIFICATION]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"
_TAGS = ["Ambon", "Indo", "Jawa"]


class IdentifikasiBahasaDataset(datasets.GeneratorBasedBuilder):
    """The "identifikasi-bahasa" dataset, manually grouped by language, \
    contains labeled Indonesian, Ambonese, and Javanese text entries, excluding \
    punctuation and numbers, with a minimum of 3,000 Ambonese, 10,000 Javanese, \
    and 3,500 Indonesian entries for effective language identification."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)
    SEACROWD_SCHEMA_NAME = "text"

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=_DATASETNAME,
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_{SEACROWD_SCHEMA_NAME}",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema=f"seacrowd_{SEACROWD_SCHEMA_NAME}",
            subset_id=_DATASETNAME,
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features({"cleanText": datasets.Value("string"), "label": datasets.Value("string")})
        elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            features = schemas.text_features(_TAGS)

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        urls = _URLS[_DATASETNAME]
        data_dir = dl_manager.download(urls)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": data_dir,
                    "split": "train",
                },
            )
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        dataset = pd.read_excel(filepath)

        if self.config.schema == "source":
            for i, row in dataset.iterrows():
                yield i, {"cleanText": row["cleanText"], "label": row["label"]}

        elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            for i, row in dataset.iterrows():
                yield i, {"id": i, "text": row["cleanText"], "label": row["label"]}



# coding=utf-8
import json
import os.path

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_DATASETNAME = "uit_viic"
_CITATION = """\
@InProceedings{10.1007/978-3-030-63007-2_57,
author="Lam, Quan Hoang
and Le, Quang Duy
and Nguyen, Van Kiet
and Nguyen, Ngan Luu-Thuy",
editor="Nguyen, Ngoc Thanh
and Hoang, Bao Hung
and Huynh, Cong Phap
and Hwang, Dosam
and Trawi{\'{n}}ski, Bogdan
and Vossen, Gottfried",
title="UIT-ViIC: A Dataset for the First Evaluation on Vietnamese Image Captioning",
booktitle="Computational Collective Intelligence",
year="2020",
publisher="Springer International Publishing",
address="Cham",
pages="730--742",
abstract="Image Captioning (IC), the task of automatic generation of image captions, has attracted
attentions from researchers in many fields of computer science, being computer vision, natural language
processing and machine learning in recent years. This paper contributes to research on Image Captioning
task in terms of extending dataset to a different language - Vietnamese. So far, there has been no existed
Image Captioning dataset for Vietnamese language, so this is the foremost fundamental step for developing
Vietnamese Image Captioning. In this scope, we first built a dataset which contains manually written
captions for images from Microsoft COCO dataset relating to sports played with balls, we called this dataset
UIT-ViIC (University Of Information Technology - Vietnamese Image Captions). UIT-ViIC consists of 19,250
Vietnamese captions for 3,850 images. Following that, we evaluated our dataset on deep neural network models
and did comparisons with English dataset and two Vietnamese datasets built by different methods. UIT-ViIC
is published on our lab website (https://sites.google.com/uit.edu.vn/uit-nlp/) for research purposes.",
isbn="978-3-030-63007-2"
}
"""

_DESCRIPTION = """
UIT-ViIC contains manually written captions for images from Microsoft COCO dataset relating to sports
played with ball. UIT-ViIC consists of 19,250 Vietnamese captions for 3,850 images. For each image,
UIT-ViIC provides five Vietnamese captions annotated by five annotators.
"""

_HOMEPAGE = "https://drive.google.com/file/d/1YexKrE6o0UiJhFWpE8M5LKoe6-k3AiM4"
_PAPER_URL = "https://arxiv.org/abs/2002.00175"
_LICENSE = Licenses.UNKNOWN.value
_HF_URL = ""
_LANGUAGES = ["vi"]
_LOCAL = False
_SUPPORTED_TASKS = [Tasks.IMAGE_CAPTIONING]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"

_URLS = "https://drive.google.com/uc?export=download&id=1YexKrE6o0UiJhFWpE8M5LKoe6-k3AiM4"
_Split_Path = {
    "train": "UIT-ViIC/uitviic_captions_train2017.json",
    "validation": "UIT-ViIC/uitviic_captions_val2017.json",
    "test": "UIT-ViIC/uitviic_captions_test2017.json",
}


class UITViICDataset(datasets.GeneratorBasedBuilder):
    BUILDER_CONFIGS = [
        SEACrowdConfig(name=f"{_DATASETNAME}_source", version=datasets.Version(_SOURCE_VERSION), description=_DESCRIPTION, subset_id=f"{_DATASETNAME}", schema="source"),
        SEACrowdConfig(name=f"{_DATASETNAME}_seacrowd_imtext", version=datasets.Version(_SEACROWD_VERSION), description=_DESCRIPTION, subset_id=f"{_DATASETNAME}", schema="seacrowd_imtext"),
    ]

    def _info(self):
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "license": datasets.Value("int32"),
                    "file_name": datasets.Value("string"),
                    "coco_url": datasets.Value("string"),
                    "flickr_url": datasets.Value("string"),
                    "height": datasets.Value("int32"),
                    "width": datasets.Value("int32"),
                    "date_captured": datasets.Value("string"),
                    "image_id": datasets.Value("int32"),
                    "caption": datasets.Value("string"),
                    "cap_id": datasets.Value("int32"),
                }
            )
        elif self.config.schema == "seacrowd_imtext":
            features = schemas.image_text_features()
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            license=_LICENSE,
            homepage=_HOMEPAGE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        file_paths = dl_manager.download_and_extract(_URLS)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": os.path.join(file_paths, _Split_Path["train"])},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"filepath": os.path.join(file_paths, _Split_Path["validation"])},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"filepath": os.path.join(file_paths, _Split_Path["test"])},
            ),
        ]

    def _generate_examples(self, filepath):
        """Yields examples."""
        with open(filepath, encoding="utf-8") as f:
            json_dict = json.load(f)
            images = {itm["id"]: itm for itm in json_dict["images"]}
            captns = json_dict["annotations"]

            for idx, capt in enumerate(captns):
                image_id = capt["image_id"]
                if self.config.schema == "source":
                    yield idx, {
                        "license": images[image_id]["license"],
                        "file_name": images[image_id]["file_name"],
                        "coco_url": images[image_id]["coco_url"],
                        "flickr_url": images[image_id]["flickr_url"],
                        "height": images[image_id]["height"],
                        "width": images[image_id]["width"],
                        "date_captured": images[image_id]["date_captured"],
                        "image_id": capt["image_id"],
                        "caption": capt["caption"],
                        "cap_id": capt["id"],
                    }
                elif self.config.schema == "seacrowd_imtext":
                    yield idx, {
                        "id": capt["id"],
                        "image_paths": [images[image_id]["coco_url"], images[image_id]["flickr_url"]],
                        "texts": capt["caption"],
                        "metadata": {
                            "context": "",
                            "labels": ["Yes"],
                        },
                    }



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
import os
import re
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks

_CITATION = """\
@misc{putri2022idk,
    doi = {10.48550/ARXIV.2210.13778},
    url = {https://arxiv.org/abs/2210.13778},
    author = {Putri, Rifki Afina and Oh, Alice},
    title = {IDK-MRC: Unanswerable Questions for Indonesian Machine Reading Comprehension},
    publisher = {arXiv},
    year = {2022}
}

"""

_LANGUAGES = ["ind"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_LOCAL = False

_ALL_DATASETS = ["idk_mrc", "trans_squad", "tydiqa", "model_gen", "human_filt"]
_DATASETNAME = _ALL_DATASETS[0]  # idk_mrc
_BASELINES = _ALL_DATASETS[1:]   # trans_squad, tydiqa, model_gen, human_filt

_DESCRIPTION = """\
I(n)dontKnow-MRC (IDK-MRC) is an Indonesian Machine Reading Comprehension dataset that covers
answerable and unanswerable questions. Based on the combination of the existing answerable questions in TyDiQA,
the new unanswerable question in IDK-MRC is generated using a question generation model and human-written question.
Each paragraph in the dataset has a set of answerable and unanswerable questions with the corresponding answer.

Besides IDK-MRC (idk_mrc) dataset, several baseline datasets also provided:
1. Trans SQuAD (trans_squad): machine translated SQuAD 2.0 (Muis and Purwarianti, 2020)
2. TyDiQA (tydiqa): Indonesian answerable questions set from the TyDiQA-GoldP (Clark et al., 2020)
3. Model Gen (model_gen): TyDiQA + the unanswerable questions output from the question generation model
4. Human Filt (human_filt): Model Gen dataset that has been filtered by human annotator
"""

_HOMEPAGE = "https://github.com/rifkiaputri/IDK-MRC"

_LICENSE = "CC-BY-SA 4.0"

_URLS = {
    _DATASETNAME: {
        "test": "https://raw.githubusercontent.com/rifkiaputri/IDK-MRC/master/dataset/idk_mrc/test.json",
        "train": "https://raw.githubusercontent.com/rifkiaputri/IDK-MRC/master/dataset/idk_mrc/train.json",
        "validation": "https://raw.githubusercontent.com/rifkiaputri/IDK-MRC/master/dataset/idk_mrc/valid.json",
    },
    "baseline": {
        "test": "https://raw.githubusercontent.com/rifkiaputri/IDK-MRC/master/dataset/baseline/{name}/test.json",
        "train": "https://raw.githubusercontent.com/rifkiaputri/IDK-MRC/master/dataset/baseline/{name}/train.json",
        "validation": "https://raw.githubusercontent.com/rifkiaputri/IDK-MRC/master/dataset/baseline/{name}/valid.json",
    },
}

_SUPPORTED_TASKS = [Tasks.QUESTION_ANSWERING]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


def seacrowd_config_constructor(name, schema, version):
    """
    Construct SEACrowdConfig with idk_mrc_{schema} format for the main dataset &
    idk_mrc_baseline_{name}_{schema} format for the baseline datasets.
    Suported dataset names: see _ALL_DATASETS
    """
    if schema != "source" and schema != "seacrowd_qa":
        raise ValueError(f"Invalid schema: {schema}")

    if name not in _ALL_DATASETS:
        raise ValueError(f"Invalid dataset name: {name}")

    if name == "idk_mrc":
        return SEACrowdConfig(
            name="idk_mrc_{schema}".format(schema=schema),
            version=datasets.Version(version),
            description="IDK-MRC with {schema} schema".format(schema=schema),
            schema=schema,
            subset_id="idk_mrc",
        )
    else:
        return SEACrowdConfig(
            name="idk_mrc_baseline_{name}_{schema}".format(name=name, schema=schema),
            version=datasets.Version(version),
            description="IDK-MRC baseline ({name}) with {schema} schema".format(name=name, schema=schema),
            schema=schema,
            subset_id="idk_mrc",
        )


class IdkMrc(datasets.GeneratorBasedBuilder):
    """IDK-MRC is an Indonesian MRC dataset that covers answerable and unanswerable questions"""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        seacrowd_config_constructor(name, schema, version)
        for name in _ALL_DATASETS for schema, version in zip(["source", "seacrowd_qa"], [_SOURCE_VERSION, _SEACROWD_VERSION])
    ]

    DEFAULT_CONFIG_NAME = "idk_mrc_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "context": datasets.Value("string"),
                    "qas": [
                        {
                            "id": datasets.Value("string"),
                            "is_impossible": datasets.Value("bool"),
                            "question": datasets.Value("string"),
                            "answers": [
                                {
                                    "text": datasets.Value("string"),
                                    "answer_start": datasets.Value("int64")
                                }
                            ]
                        }
                    ],
                }
            )

        elif self.config.schema == "seacrowd_qa":
            features = schemas.qa_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        if self.config.name == "idk_mrc_source" or self.config.name == "idk_mrc_seacrowd_qa":
            data_name = "idk_mrc"
            train_data_path = dl_manager.download_and_extract(_URLS[_DATASETNAME]["train"])
            validation_data_path = dl_manager.download_and_extract(_URLS[_DATASETNAME]["validation"])
            test_data_path = dl_manager.download_and_extract(_URLS[_DATASETNAME]["test"])
        else:
            try:
                data_name = re.search("baseline_(.+?)_(source|seacrowd_qa)", self.config.name).group(1)
            except AttributeError:
                raise ValueError(f"Invalid config name: {self.config.name}")

            if data_name not in _BASELINES:
                raise ValueError(f"Invalid baseline dataset name: {data_name}")

            train_data_path = dl_manager.download_and_extract(_URLS["baseline"]["train"].format(name=data_name))
            validation_data_path = dl_manager.download_and_extract(_URLS["baseline"]["validation"].format(name=data_name))
            test_data_path = dl_manager.download_and_extract(_URLS["baseline"]["test"].format(name=data_name)) if data_name != "trans_squad" else ""

        data_split = [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": train_data_path,
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": os.path.join(validation_data_path),
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": os.path.join(test_data_path),
                },
            ),
        ]

        if data_name == "trans_squad":
            # trans_squad doesn't have test split
            return data_split[:2]

        return data_split

    def _generate_examples(self, filepath: Path) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        with open(filepath) as json_file:
            examples = json.load(json_file)

        if self.config.schema == "source":
            # Dataset doesn't have predefined context ID, use python enumeration.
            # The examples in the source schema are grouped by paragraph context;
            # each context can have multiple questions.
            for key, example in enumerate(examples):
                yield key, example

        elif self.config.schema == "seacrowd_qa":
            for key, example in enumerate(examples):
                for qa in example["qas"]:
                    # Use question ID as key
                    yield str(qa["id"]), {
                        "id": qa["id"],
                        "question_id": qa["id"],
                        "document_id": str(key),
                        "question": qa["question"],
                        "type": "extractive",
                        "choices": [],
                        "context": example["context"],
                        "answer": [ans["text"] for ans in qa["answers"]],
                        "meta": {}
                    }



from pathlib import Path
from typing import Dict, List, Tuple

import datasets
import pandas as pd

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks

_CITATION = """\
@inproceedings{ibrohim-budi-2019-multi,
    title = "Multi-label Hate Speech and Abusive Language Detection in {I}ndonesian {T}witter",
    author = "Ibrohim, Muhammad Okky  and
      Budi, Indra",
    booktitle = "Proceedings of the Third Workshop on Abusive Language Online",
    month = aug,
    year = "2019",
    address = "Florence, Italy",
    publisher = "Association for Computational Linguistics",
    url = "https://aclanthology.org/W19-3506",
    doi = "10.18653/v1/W19-3506",
    pages = "46--57",
}
"""

_LOCAL = False
_LANGUAGES = ["ind"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_DATASETNAME = "id_multilabel_hs"

_DESCRIPTION = """\
The ID_MULTILABEL_HS dataset is collection of 13,169 tweets in Indonesian language,
designed for hate speech detection NLP task. This dataset is combination from previous research and newly crawled data from Twitter.
This is a multilabel dataset with label details as follows:
-HS : hate speech label;
-Abusive : abusive language label;
-HS_Individual : hate speech targeted to an individual;
-HS_Group : hate speech targeted to a group;
-HS_Religion : hate speech related to religion/creed;
-HS_Race : hate speech related to race/ethnicity;
-HS_Physical : hate speech related to physical/disability;
-HS_Gender : hate speech related to gender/sexual orientation;
-HS_Gender : hate related to other invective/slander;
-HS_Weak : weak hate speech;
-HS_Moderate : moderate hate speech;
-HS_Strong : strong hate speech.
"""

_HOMEPAGE = "https://aclanthology.org/W19-3506/"
_LICENSE = "Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International"
_URLS = {
    _DATASETNAME: "https://raw.githubusercontent.com/okkyibrohim/id-multi-label-hate-speech-and-abusive-language-detection/master/re_dataset.csv",
}
_SUPPORTED_TASKS = [Tasks.ASPECT_BASED_SENTIMENT_ANALYSIS]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class IdAbusive(datasets.GeneratorBasedBuilder):
    """The ID_MULTILABEL_HS dataset is multi-label hate speech and abusive language detection in Indonesian tweets"""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="id_multilabel_hs_source",
            version=SOURCE_VERSION,
            description="ID Multilabel HS source schema",
            schema="source",
            subset_id="id_multilabel_hs",
        ),
        SEACrowdConfig(
            name="id_multilabel_hs_seacrowd_text_multi",
            version=SEACROWD_VERSION,
            description="ID Multilabel HS Nusantara schema",
            schema="seacrowd_text_multi",
            subset_id="id_multilabel_hs",
        ),
    ]

    DEFAULT_CONFIG_NAME = "id_multilabel_hs_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features({
                "tweet": datasets.Value("string"), 
                "HS": datasets.Value("bool"),
                "Abusive": datasets.Value("bool"), 
                "HS_Individual": datasets.Value("bool"), 
                "HS_Group": datasets.Value("bool"), 
                "HS_Religion": datasets.Value("bool"), 
                "HS_Race": datasets.Value("bool"), 
                "HS_Physical": datasets.Value("bool"), 
                "HS_Gender": datasets.Value("bool"), 
                "HS_Other": datasets.Value("bool"), 
                "HS_Weak": datasets.Value("bool"), 
                "HS_Moderate": datasets.Value("bool"), 
                "HS_Strong": datasets.Value("bool"),
                })
        elif self.config.schema == "seacrowd_text_multi":
            features = schemas.text_multi_features([0, 1])

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        # Dataset does not have predetermined split, putting all as TRAIN
        urls = _URLS[_DATASETNAME]
        base_dir = Path(dl_manager.download_and_extract(urls))
        data_files = {"train": base_dir}

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": data_files["train"],
                    "split": "train",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        # Dataset does not have id, using row index as id
        label_cols = ["HS", "Abusive", "HS_Individual", "HS_Group", "HS_Religion", "HS_Race", "HS_Physical", "HS_Gender", "HS_Other", "HS_Weak", "HS_Moderate", "HS_Strong"]
        df = pd.read_csv(filepath, encoding="ISO-8859-1").reset_index()
        df.columns = ["id", "tweet"] + label_cols

        if self.config.schema == "source":
            for row in df.itertuples():
                ex = {
                    "tweet": row.tweet,
                }
                for label in label_cols:
                    ex[label] = getattr(row, label)
                yield row.id, ex

        elif self.config.schema == "seacrowd_text_multi":
            for row in df.itertuples():
                ex = {
                    "id": str(row.id),
                    "text": row.tweet,
                    "labels": [label for label in row[3:]],
                }
                yield row.id, ex
        else:
            raise ValueError(f"Invalid config: {self.config.name}")



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import csv
import os
from pathlib import Path
from typing import List

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks

_CITATION = """\
@inproceedings{kjartansson-etal-sltu2018,
    title = {{Crowd-Sourced Speech Corpora for Javanese, Sundanese,  Sinhala, Nepali, and Bangladeshi Bengali}},
    author = {Oddur Kjartansson and Supheakmungkol Sarin and Knot Pipatsrisawat and Martin Jansche and Linne Ha},
    booktitle = {Proc. The 6th Intl. Workshop on Spoken Language Technologies for Under-Resourced Languages (SLTU)},
    year  = {2018},
    address = {Gurugram, India},
    month = aug,
    pages = {52--55},
    URL   = {http://dx.doi.org/10.21437/SLTU.2018-11},
  }
"""

_DATASETNAME = "jv_id_asr"

_DESCRIPTION = """\
This data set contains transcribed audio data for Javanese. The data set consists of wave files, and a TSV file.
The file utt_spk_text.tsv contains a FileID, UserID and the transcription of audio in the file.
The data set has been manually quality checked, but there might still be errors.
This dataset was collected by Google in collaboration with Reykjavik University and Universitas Gadjah Mada in Indonesia.
"""

_HOMEPAGE = "http://openslr.org/35/"
_LANGUAGES = ["jav"]
_LOCAL = False

_LICENSE = "Attribution-ShareAlike 4.0 International"

_URLS = {
    _DATASETNAME: "https://www.openslr.org/resources/35/asr_javanese_{}.zip",
}

_SUPPORTED_TASKS = [Tasks.SPEECH_RECOGNITION]  # example: [Tasks.TRANSLATION, Tasks.NAMED_ENTITY_RECOGNITION, Tasks.RELATION_EXTRACTION]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class JvIdASR(datasets.GeneratorBasedBuilder):
    """Javanese ASR training data set containing ~185K utterances."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="jv_id_asr_source",
            version=SOURCE_VERSION,
            description="jv_id_asr source schema",
            schema="source",
            subset_id="jv_id_asr",
        ),
        SEACrowdConfig(
            name="jv_id_asr_seacrowd_sptext",
            version=SEACROWD_VERSION,
            description="jv_id_asr Nusantara schema",
            schema="seacrowd_sptext",
            subset_id="jv_id_asr",
        ),
    ]

    DEFAULT_CONFIG_NAME = "jv_id_asr_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "speaker_id": datasets.Value("string"),
                    "path": datasets.Value("string"),
                    "audio": datasets.Audio(sampling_rate=16_000),
                    "text": datasets.Value("string"),
                }
            )
        elif self.config.schema == "seacrowd_sptext":
            features = schemas.speech_text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        urls = _URLS[_DATASETNAME]
        base_path = {}
        for id in range(10):
            base_path[id] = dl_manager.download_and_extract(urls.format(str(id)))
        for id in ["a", "b", "c", "d", "e", "f"]:
            base_path[id] = dl_manager.download_and_extract(urls.format(str(id)))
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": base_path},
            ),
        ]

    def _generate_examples(self, filepath: Path):
        for key, fp in filepath.items():
            tsv_file = os.path.join(fp, "asr_javanese", "utt_spk_text.tsv")
            with open(tsv_file, "r") as f:
                tsv_file = csv.reader(f, delimiter="\t")
                for line in tsv_file:
                    audio_id, sp_id, text = line[0], line[1], line[2]
                    wav_path = os.path.join(fp, "asr_javanese", "data", "{}".format(audio_id[:2]), "{}.flac".format(audio_id))

                    if os.path.exists(wav_path):
                        if self.config.schema == "source":
                            ex = {
                                "id": audio_id,
                                "speaker_id": sp_id,
                                "path": wav_path,
                                "audio": wav_path,
                                "text": text,
                            }
                            yield audio_id, ex
                        elif self.config.schema == "seacrowd_sptext":
                            ex = {
                                "id": audio_id,
                                "speaker_id": sp_id,
                                "path": wav_path,
                                "audio": wav_path,
                                "text": text,
                                "metadata": {
                                    "speaker_age": None,
                                    "speaker_gender": None,
                                },
                            }
                            yield audio_id, ex
            f.close()



import os
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@inproceedings{PhoMT,
title     = {{PhoMT: A High-Quality and Large-Scale Benchmark Dataset for Vietnamese-English Machine Translation}},
author    = {Long Doan and Linh The Nguyen and Nguyen Luong Tran and Thai Hoang and Dat Quoc Nguyen},
booktitle = {Proceedings of the 2021 Conference on Empirical Methods in Natural Language Processing},
year      = {2021},
pages     = {4495--4503}
}
"""

_DATASETNAME = "phomt"


_DESCRIPTION = """\
PhoMT is a high-quality and large-scale Vietnamese-English parallel dataset of 3.02M sentence pairs, which is 2.9M
pairs larger than the benchmark Vietnamese-English machine translation corpus IWSLT15. This is the first large-scale
Vietnamese-English machine translation study.
"""

_LANGUAGES = ["vie", "eng"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_LOCAL = True

_HOMEPAGE = "https://github.com/VinAIResearch/PhoMT"

_LICENSE = Licenses.MIT.value

_SUPPORTED_TASKS = [Tasks.MACHINE_TRANSLATION]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"

MAP_LANG = {"eng": "en", "vie": "vi"}


def seacrowd_config_constructor(src_lang, tgt_lang, schema, version):
    if src_lang == "" or tgt_lang == "":
        raise ValueError(f"Invalid src_lang {src_lang} or tgt_lang {tgt_lang}")

    if schema not in ["source", "seacrowd_t2t"]:
        raise ValueError(f"Invalid schema: {schema}")

    return SEACrowdConfig(
        name="phomt_{src}_{tgt}_{schema}".format(src=src_lang, tgt=tgt_lang, schema=schema),
        version=datasets.Version(version),
        description="phomt schema for {schema} from {src} to {tgt}".format(schema=schema, src=src_lang, tgt=tgt_lang),
        schema=schema,
        subset_id="phomt_{src}_{tgt}".format(src=src_lang, tgt=tgt_lang),
    )


class PhoMT(datasets.GeneratorBasedBuilder):
    """
    PhoMT is a high-quality and large-scale Vietnamese-English parallel dataset of 3.02M sentence pairs, which is
    2.9M pairs larger than the benchmark Vietnamese-English machine translation corpus IWSLT15.
    """

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        seacrowd_config_constructor("eng", "vie", "source", _SOURCE_VERSION),
        seacrowd_config_constructor("eng", "vie", "seacrowd_t2t", _SEACROWD_VERSION),
    ]

    DEFAULT_CONFIG_NAME = "phomt_eng_vie_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema in ("source", "seacrowd_t2t"):
            features = schemas.text2text_features
        else:
            raise ValueError(f"Invalid config schema: {self.config.schema}")

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        if self.config.data_dir is None:
            raise ValueError("This is a local dataset. Please pass the data_dir kwarg to load_dataset.")
        else:
            data_dir = self.config.data_dir

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": os.path.join(data_dir, "detokenization", "train", "train.{lang}")},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"filepath": os.path.join(data_dir, "detokenization", "dev", "dev.{lang}")},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"filepath": os.path.join(data_dir, "detokenization", "test", "test.{lang}")},
            ),
        ]

    def _generate_examples(self, filepath: Path) -> Tuple[int, Dict]:
        config_names_split = self.config.name.split("_")
        src_lang = config_names_split[1]
        tgt_lang = config_names_split[2]

        src_path = filepath.format(lang=MAP_LANG[src_lang])
        tgt_path = filepath.format(lang=MAP_LANG[tgt_lang])

        with open(src_path, "r", encoding="utf8") as f:
            src_lines = f.readlines()
        with open(tgt_path, "r", encoding="utf8") as f:
            tgt_lines = f.readlines()

        if self.config.schema in ("source", "seacrowd_t2t"):
            for idx, (src_line, tgt_line) in enumerate(zip(src_lines, tgt_lines)):
                ex = {
                    "id": str(idx),
                    "text_1": src_line.strip(),
                    "text_2": tgt_line.strip(),
                    "text_1_name": src_lang,
                    "text_2_name": tgt_lang,
                }
                yield idx, ex

        else:
            raise NotImplementedError(f"Schema '{self.config.schema}' is not defined.")



from pathlib import Path
from typing import Dict, List, Tuple

import datasets
import pandas as pd

from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@inproceedings{
    mukherjee-etal-2023-global,
    title = "{G}lobal {V}oices, Local Biases: Socio-Cultural Prejudices across Languages",
    author = "Mukherjee, Anjishnu and Raj, Chahat and Zhu, Ziwei and Anastasopoulos, Antonios",
    editor = "Bouamor, Houda and Pino, Juan and Bali, Kalika",
    booktitle = "Proceedings of the 2023 Conference on Empirical Methods in Natural Language Processing",
    month = dec,
    year = "2023",
    address = "Singapore",
    publisher = "Association for Computational Linguistics",
    url = "https://aclanthology.org/2023.emnlp-main.981",
    doi = "10.18653/v1/2023.emnlp-main.981",
    pages = "15828--15845",
    abstract = "Human biases are ubiquitous but not uniform: disparities exist across linguistic, cultural, and societal borders. As large amounts of recent literature suggest, language models (LMs) trained on human data can reflect and often amplify the effects of these social biases. However, the vast majority of existing studies on bias are heavily skewed towards Western and European languages. In this work, we scale the Word Embedding Association Test (WEAT) to 24 languages, enabling broader studies and yielding interesting findings about LM bias. We additionally enhance this data with culturally relevant information for each language, capturing local contexts on a global scale. Further, to encompass more widely prevalent societal biases, we examine new bias dimensions across toxicity, ableism, and more. Moreover, we delve deeper into the Indian linguistic landscape, conducting a comprehensive regional bias analysis across six prevalent Indian languages. Finally, we highlight the significance of these social biases and the new dimensions through an extensive comparison of embedding methods, reinforcing the need to address them in pursuit of more equitable language models."
}
"""

_DATASETNAME = "weathub"

_DESCRIPTION = """\
    WEATHub is a dataset containing 24 languages. It contains words organized into groups of (target1, target2, attribute1, attribute2) to measure the association target1:target2 :: attribute1:attribute2. For example target1 can be insects, target2 can be flowers. And we might be trying to measure whether we find insects or flowers pleasant or unpleasant. The measurement of word associations is quantified using the WEAT metric from their paper. It is a metric that calculates an effect size (Cohen's d) and also provides a p-value (to measure statistical significance of the results). In their paper, they use word embeddings from language models to perform these tests and understand biased associations in language models across different languages.
"""

_HOMEPAGE = "https://huggingface.co/datasets/iamshnoo/WEATHub"

_LANGUAGES = ["tha", "tgl", "vie", "cmn", "eng"]

_LICENSE = Licenses.CC_BY_4_0.value

_LOCAL = False

_URLS = {
    "original_weat": "https://huggingface.co/datasets/iamshnoo/WEATHub/resolve/main/data/original_weat-00000-of-00001-7e85a83920777d87.parquet",
    "new_human_biases": "https://huggingface.co/datasets/iamshnoo/WEATHub/resolve/main/data/new_human_biases-00000-of-00001-557150d939938864.parquet",
    "india_specific_biases": "https://huggingface.co/datasets/iamshnoo/WEATHub/resolve/main/data/india_specific_biases-00000-of-00001-a2b8ed74878afc1b.parquet",
}

_SUPPORTED_TASKS = [Tasks.WORD_LIST]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class WEATHubDataset(datasets.GeneratorBasedBuilder):
    """WEATHub is a dataset containing 24 languages. It contains words organized into groups of (target1, target2, attribute1, attribute2) to measure the association target1:target2 :: attribute1:attribute2. This dataset corresponds to the data described in the paper "Global Voices, Local Biases: Socio-Cultural Prejudices across Languages" accepted to EMNLP 2023."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)
    SEA_LANGUAGES = set(["th", "tl", "vi", "zh", "en"])

    subsets = [
        f"{_DATASETNAME}.original_weat",
        f"{_DATASETNAME}.new_human_biases",
        f"{_DATASETNAME}.india_specific_biases",
    ]

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{sub}_source",
            version=datasets.Version(_SOURCE_VERSION),
            description=f"{sub} source schema",
            schema="source",
            subset_id=sub,
        )
        for sub in subsets
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}.original_weat_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "language": datasets.Value("string"),
                    "weat": datasets.Value("string"),
                    "attr1.category": datasets.Value("string"),
                    "attr1.examples": [datasets.Value("string")],
                    "attr2.category": datasets.Value("string"),
                    "attr2.examples": [datasets.Value("string")],
                    "targ1.category": datasets.Value("string"),
                    "targ1.examples": [datasets.Value("string")],
                    "targ2.category": datasets.Value("string"),
                    "targ2.examples": [datasets.Value("string")],
                }
            )
        elif "seacrowd" in self.config.schema:
            raise NotImplementedError("No seacrowd schema for word list tasks")
        else:
            raise ValueError("Invalid schema name")

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        urls = _URLS
        data_dir = {split: dl_manager.download_and_extract(url) for split, url in urls.items()}

        subset = "_".join(self.config.name.split(".")[-1].split("_")[:-1])

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": data_dir[subset],
                },
            ),
        ]

    def _generate_examples(self, filepath: Path) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""

        if self.config.schema == "source":
            df = pd.read_parquet(filepath)
            for i, example in df.iterrows():
                if example["language"] not in self.SEA_LANGUAGES:
                    continue
                yield str(i), {
                    "language": example["language"],
                    "weat": example["weat"],
                    "attr1.category": example["attr1.category"],
                    "attr1.examples": example["attr1.examples"],
                    "attr2.category": example["attr2.category"],
                    "attr2.examples": example["attr2.examples"],
                    "targ1.category": example["targ1.category"],
                    "targ1.examples": example["targ1.examples"],
                    "targ2.category": example["targ2.category"],
                    "targ2.examples": example["targ2.examples"],
                }



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.common_parser import load_conll_data
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks

_CITATION = """\
@INPROCEEDINGS{8355036,
  author={Alfina, Ika and Savitri, Septiviana and Fanany, Mohamad Ivan},
  title={Modified DBpedia entities expansion for tagging automatically NER dataset},
  booktitle={2017 International Conference on Advanced Computer Science and Information Systems (ICACSIS)},
  pages={216-221},
  year={2017},
  url={https://ieeexplore.ieee.org/document/8355036},
  doi={10.1109/ICACSIS.2017.8355036}}

@INPROCEEDINGS{7872784,
  author={Alfina, Ika and Manurung, Ruli and Fanany, Mohamad Ivan},
  booktitle={2016 International Conference on Advanced Computer Science and Information Systems (ICACSIS)},
  title={DBpedia entities expansion in automatically building dataset for Indonesian NER},
  year={2016},
  pages={335-340},
  doi={10.1109/ICACSIS.2016.7872784}}
"""

_LOCAL = False
_LANGUAGES = ["ind"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_DATASETNAME = "singgalang"

_DESCRIPTION = """\
Rule-based annotation Indonesian NER Dataset of 48,957 sentences or 1,478,286 tokens.
Annotation conforms the Stanford-NER format (https://stanfordnlp.github.io/CoreNLP/ner.html) for 3 NER tags of Person, Organisation, and Place.
This dataset consists of 41,297, 14,770, and 82,179 tokens of entity (respectively) from over 14, 6, and 5 rules.
"""

_HOMEPAGE = "https://github.com/ir-nlp-csui/singgalang"

_LICENSE = """\
You can use this dataset for free. You don't need our permission to use it. Please cite our paper if your work uses our data in your publication.
Please note that you are not allowed to create a copy of this dataset and share it publicly in your own repository without our permission.\
"""

_URLS = {
    _DATASETNAME: "https://raw.githubusercontent.com/ir-nlp-csui/singgalang/main/SINGGALANG.tsv",
}

_SUPPORTED_TASKS = [Tasks.NAMED_ENTITY_RECOGNITION]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class SinggalangDataset(datasets.GeneratorBasedBuilder):
    """Rule-based annotation Indonesian NER Dataset of 48,957 sentences with 3 NER tags"""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    label_classes = [
        "O",
        "Person",
        "Organisation",
        "Place",
    ]

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_seq_label",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} Nusantara schema",
            schema="seacrowd_seq_label",
            subset_id=f"{_DATASETNAME}",
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:

        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "sentence": [datasets.Value("string")],
                    "label": [datasets.Value("string")],
                }
            )

        elif self.config.schema == "seacrowd_seq_label":
            features = schemas.seq_label_features(self.label_classes)

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        url = _URLS[_DATASETNAME]
        data_path = dl_manager.download(url)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": data_path,
                },
            ),
        ]

    def _generate_examples(self, filepath: Path) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""

        dataset = load_conll_data(filepath)

        if self.config.schema == "source":
            for key, ex in enumerate(dataset):
                yield key, ex

        elif self.config.schema == "seacrowd_seq_label":
            for key, ex in enumerate(dataset):
                yield key, {
                    "id": str(key),
                    "tokens": ex["sentence"],
                    "labels": ex["label"],
                }



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import os
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.sea_datasets.alt_burmese_treebank.utils.alt_burmese_treebank_utils import extract_data
from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@article{
    10.1145/3373268,
    author = {Ding, Chenchen and Yee, Sann Su Su and Pa, Win Pa and Soe, Khin Mar and Utiyama, Masao and Sumita, Eiichiro},
    title = {A Burmese (Myanmar) Treebank: Guideline and Analysis},
    year = {2020},
    issue_date = {May 2020},
    publisher = {Association for Computing Machinery},
    address = {New York, NY, USA},
    volume = {19},
    number = {3},
    issn = {2375-4699},
    url = {https://doi.org/10.1145/3373268},
    doi = {10.1145/3373268},
    abstract = {A 20,000-sentence Burmese (Myanmar) treebank on news articles has been released under a CC BY-NC-SA license.\
               Complete phrase structure annotation was developed for each sentence from the morphologically annotated data\
               prepared in previous work of Ding et&nbsp;al. [1]. As the final result of the Burmese component in the Asian\
               Language Treebank Project, this is the first large-scale, open-access treebank for the Burmese language.\
               The annotation details and features of this treebank are presented.\
               },
    journal = {ACM Trans. Asian Low-Resour. Lang. Inf. Process.},
    month = {jan},
    articleno = {40},
    numpages = {13},
    keywords = {Burmese (Myanmar), phrase structure, treebank}
}
"""

_DATASETNAME = "alt_burmese_treebank"

_DESCRIPTION = """\
A 20,000-sentence Burmese (Myanmar) treebank on news articles containing complete phrase structure annotation.\
As the final result of the Burmese component in the Asian Language Treebank Project, this is the first large-scale,\
open-access treebank for the Burmese language.
"""

_HOMEPAGE = "https://zenodo.org/records/3463010"

_LANGUAGES = ["mya"]

_LICENSE = Licenses.CC_BY_NC_SA_4_0.value

_LOCAL = False

_URLS = {
    _DATASETNAME: "https://zenodo.org/records/3463010/files/my-alt-190530.zip?download=1",
}

_SUPPORTED_TASKS = [Tasks.CONSTITUENCY_PARSING]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class AltBurmeseTreebank(datasets.GeneratorBasedBuilder):
    """A 20,000-sentence Burmese (Myanmar) treebank on news articles containing complete phrase structure annotation.\
       As the final result of the Burmese component in the Asian Language Treebank Project, this is the first large-scale,\
       open-access treebank for the Burmese language."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_tree",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema="seacrowd_tree",
            subset_id=f"{_DATASETNAME}",
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:

        if self.config.schema == "source":
            features = datasets.Features({"id": datasets.Value("string"), "text": datasets.Value("string")})
        elif self.config.schema == "seacrowd_tree":
            features = schemas.tree_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        urls = _URLS[_DATASETNAME]
        data_dir = dl_manager.download_and_extract(urls)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": os.path.join(data_dir, "my-alt-190530/data"),
                    "split": "train",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""

        if self.config.schema == "source":
            with open(filepath, "r") as f:
                for idx, line in enumerate(f):
                    example = {"id": line.split("\t")[0], "text": line.split("\t")[1]}
                    yield idx, example

        elif self.config.schema == "seacrowd_tree":
            with open(filepath, "r") as f:
                for idx, line in enumerate(f):
                    example = extract_data(line)
                    yield idx, example



"""
Unit-tests to ensure tasks adhere to seacrowd schema.
"""
import argparse
import importlib
import sys
import unittest
from collections import defaultdict
from pathlib import Path
from typing import Iterable, Iterator, List, Optional, Union, Dict

import datasets
from datasets import DatasetDict, Features
from seacrowd.utils.constants import Tasks, TASK_TO_SCHEMA, VALID_TASKS, VALID_SCHEMAS, SCHEMA_TO_FEATURES, TASK_TO_FEATURES
from seacrowd.utils.schemas import kb_features, pairs_features, pairs_features_score, qa_features, text2text_features, text_features, text_multi_features, seq_label_features, ssp_features, speech_text_features, image_text_features

sys.path.append(str(Path(__file__).parent.parent))

import logging

logger = logging.getLogger(__name__)


def _get_example_text(example: dict) -> str:
    """
    Concatenate all text from passages in an example of a KB schema
    :param example: An instance of the KB schema
    """
    return " ".join([t for p in example["passages"] for t in p["text"]])


OFFSET_ERROR_MSG = "\n\n" "There are features with wrong offsets!" " This is not a hard failure, as it is common for this type of datasets." " However, if the error list is long (e.g. >10) you should double check your code. \n\n"


class TestDataLoader(unittest.TestCase):
    """
    Given a dataset script that has been implemented, check if it adheres to the `seacrowd` schema.

    The test
    """

    PATH: str
    SCHEMA: str
    SUBSET_ID: str
    DATA_DIR: Optional[str]
    USE_AUTH_TOKEN: Optional[Union[bool, str]]

    def runTest(self):
        """
         Run all tests that check:
         (1) [removed - full path to script is now passed in] test_name: Checks if
             dataloading script has correct path format
         (2) setUp: Checks data and _SUPPORTED_TASKS can be loaded
         (3) print_statistics: Counts number of all possible schema keys/instances of the examples
         (4) test_schema: confirms seacrowd keys present
         (5) test_are_ids_globally_unique: Checks if all examples have a unique identifier

         # KB-Specific tests
         (6) test_do_all_referenced_ids_exist: Check if any sub-key (ex: entities/events etc.)
             have referenced keys
         (7) test_passages_offsets: Check if text matches offsets in passages
         (8) test_entities_offsets: Check if text matches offsets in entities
         (9) test_events_offsets: Check if text matches offsets in events
        (10) test_coref_ids: Check if text matches offsets in coreferences

        """  # noqa
        pass


    def setUp(self) -> None:
        """Load original and seacrowd schema views"""

        logger.info(f"self.PATH: {self.PATH}")
        logger.info(f"self.SUBSET_ID: {self.SUBSET_ID}")
        logger.info(f"self.SCHEMA: {self.SCHEMA}")
        logger.info(f"self.DATA_DIR: {self.DATA_DIR}")

        # Get task type of the dataset
        logger.info("Checking for _SUPPORTED_TASKS ...")
        module = self.PATH
        if module.endswith(".py"):
            module = module[:-3]
        module = module.replace("/", ".")
        print("module", module)
        self._SUPPORTED_TASKS = importlib.import_module(module)._SUPPORTED_TASKS
        logger.info(f"Found _SUPPORTED_TASKS={self._SUPPORTED_TASKS}")
        invalid_tasks = set(self._SUPPORTED_TASKS) - VALID_TASKS
        if len(invalid_tasks) > 0:
            raise ValueError(f"Found invalid supported tasks {invalid_tasks}. Must be one of {VALID_TASKS}")

        config_name = f"{self.SUBSET_ID}_source"
        logger.info(f"Checking load_dataset with config name {config_name}")
        self.dataset_source = datasets.load_dataset(
            self.PATH,
            name=config_name,
            data_dir=self.DATA_DIR,
            use_auth_token=self.USE_AUTH_TOKEN,
        )

        # check dataset samples
        for schema in ['source']:
            dataset = datasets.load_dataset(
                self.PATH,
                name=f"{self.SUBSET_ID}_{schema}",
                data_dir=self.DATA_DIR,
                use_auth_token=self.USE_AUTH_TOKEN,
            )
            logger.info(f"Dataset sample [{schema}]\n{dataset[list(dataset.keys())[0]][0]}")


    def get_feature_statistics(self, features: Features, schema: str) -> Dict:
        """
        Gets sample statistics, for each split and sample of the number of
        features in the schema present; only works for the seacrowd schema.

        :param schema_type: Type of schema to reference features from
        """  # noqa
        logger.info("Gathering schema statistics")
        all_counters = {}
        for split_name, split in self.datasets_seacrowd[schema].items():

            counter = defaultdict(int)
            for example in split:
                for feature_name, feature in features.items():
                    if example.get(feature_name, None) is not None:
                        if isinstance(feature, datasets.ClassLabel) or isinstance(feature, datasets.Value):
                            if example[feature_name] is not None:
                                counter[feature_name] += 1
                        else:
                            counter[feature_name] += len(example[feature_name])

                            # TODO do proper recursion here
                            if feature_name == "entities":
                                for entity in example["entities"]:
                                    counter["normalized"] += len(entity["normalized"])

            all_counters[split_name] = counter

        return all_counters

    def _assert_ids_globally_unique(
        self,
        collection: Iterable,
        ids_seen: set,
        ignore_assertion: bool = False,
    ):
        """
        Checks if all IDs are globally unique across a feature list.
        This looks recursively through elements of arrays to check if every referenced ID is unique.

        :param collection: An iterable of features that contain NLP info (ex: entities, events)
        :param ids_seen: Set of previously seen numerical IDs (empty by default)
        :param ignore_assertion: Whether to raise an error if id was already seen.
        """  # noqa
        if isinstance(collection, dict):

            for k, v in collection.items():
                if isinstance(v, dict):
                    self._assert_ids_globally_unique(v, ids_seen)

                elif isinstance(v, list):
                    for elem in v:
                        self._assert_ids_globally_unique(elem, ids_seen)
                else:
                    if k == "id":
                        if not ignore_assertion:
                            self.assertNotIn(v, ids_seen)
                        ids_seen.add(v)

        elif isinstance(collection, list):
            for elem in collection:
                self._assert_ids_globally_unique(elem, ids_seen)

    def test_are_ids_globally_unique(self, dataset_seacrowd: DatasetDict):
        """
        Tests each example in a split has a unique ID.
        """
        logger.info("Checking global ID uniqueness")
        for split in dataset_seacrowd.values():
            ids_seen = set()
            for example in split:
                self._assert_ids_globally_unique(example, ids_seen=ids_seen)
        logger.info("Found {} unique IDs".format(len(ids_seen)))

    def _get_referenced_ids(self, example):
        referenced_ids = []

        if example.get("events", None) is not None:
            for event in example["events"]:
                for argument in event["arguments"]:
                    referenced_ids.append((argument["ref_id"], "event"))

        if example.get("coreferences", None) is not None:
            for coreference in example["coreferences"]:
                for entity_id in coreference["entity_ids"]:
                    referenced_ids.append((entity_id, "entity"))

        if example.get("relations", None) is not None:
            for relation in example["relations"]:
                referenced_ids.append((relation["arg1_id"], "entity"))
                referenced_ids.append((relation["arg2_id"], "entity"))

        return referenced_ids

    def _get_existing_referable_ids(self, example):
        existing_ids = []

        for entity in example["entities"]:
            existing_ids.append((entity["id"], "entity"))

        if example.get("events", None) is not None:
            for event in example["events"]:
                existing_ids.append((event["id"], "event"))

        return existing_ids

    def test_do_all_referenced_ids_exist(self, dataset_seacrowd: DatasetDict):
        """
        Checks if referenced IDs are correctly labeled.
        """
        logger.info("Checking if referenced IDs are properly mapped")
        for split in dataset_seacrowd.values():
            for example in split:
                referenced_ids = set()
                existing_ids = set()

                referenced_ids.update(self._get_referenced_ids(example))
                existing_ids.update(self._get_existing_referable_ids(example))

                for ref_id, ref_type in referenced_ids:
                    if ref_type == "event":
                        if not ((ref_id, "entity") in existing_ids or (ref_id, "event") in existing_ids):
                            logger.warning(f"Referenced element ({ref_id}, entity/event) could not be found in existing ids {existing_ids}. Please make sure that this is not because of a bug in your data loader.")
                    else:
                        if not (ref_id, ref_type) in referenced_ids:
                            logger.warning(f"Referenced element {(ref_id, ref_type)} could not be found in existing ids {existing_ids}. Please make sure that this is not because of a bug in your data loader.")

    def test_passages_offsets(self, dataset_seacrowd: DatasetDict):
        """
        Verify that the passages offsets are correct,
        i.e.: passage text == text extracted via the passage offsets
        """  # noqa
        logger.info("KB ONLY: Checking passage offsets")
        for split in dataset_seacrowd:

            if "passages" in dataset_seacrowd[split].features:

                for example in dataset_seacrowd[split]:

                    example_text = _get_example_text(example)

                    for passage in example["passages"]:

                        example_id = example["id"]

                        text = passage["text"]
                        offsets = passage["offsets"]

                        self._test_is_list(msg="Text in passages must be a list", field=text)

                        self._test_is_list(
                            msg="Offsets in passages must be a list",
                            field=offsets,
                        )

                        self._test_has_only_one_item(
                            msg="Offsets in passages must have only one element",
                            field=offsets,
                        )

                        self._test_has_only_one_item(
                            msg="Text in passages must have only one element",
                            field=text,
                        )

                        for idx, (start, end) in enumerate(offsets):
                            msg = f"Split:{split} - Example:{example_id} - text:`{example_text[start:end]}` != text_by_offset:`{text[idx]}`"
                            self.assertEqual(example_text[start:end], text[idx], msg)

    def _check_offsets(
        self,
        example_id: int,
        split: str,
        example_text: str,
        offsets: List[List[int]],
        texts: List[str],
    ) -> Iterator:
        """

        :param example_text:
        :param offsets:
        :param texts:

        """  # noqa

        if len(texts) != len(offsets):
            logger.warning(f"Split:{split} - Example:{example_id} - Number of texts {len(texts)} != number of offsets {len(offsets)}. Please make sure that this error already exists in the original data and was not introduced in the data loader.")

        self._test_is_list(
            msg=f"Split:{split} - Example:{example_id} - Text fields paired with offsets must be in the form [`text`, ...]",
            field=texts,
        )

        with self.subTest(
            f"Split:{split} - Example:{example_id} - All offsets must be in the form [(lo1, hi1), ...]",
            offsets=offsets,
        ):
            self.assertTrue(all(len(o) == 2 for o in offsets))

        # offsets are always list of lists
        for idx, (start, end) in enumerate(offsets):

            by_offset_text = example_text[start:end]
            try:
                text = texts[idx]
            except IndexError:
                text = ""

            if by_offset_text != text:
                yield f" text:`{text}` != text_by_offset:`{by_offset_text}`"

    def test_entities_offsets(self, dataset_seacrowd: DatasetDict):
        """
        Verify that the entities offsets are correct,
        i.e.: entity text == text extracted via the entity offsets
        """  # noqa
        logger.info("KB ONLY: Checking entity offsets")
        errors = []

        for split in dataset_seacrowd:

            if "entities" in dataset_seacrowd[split].features:

                for example in dataset_seacrowd[split]:

                    example_id = example["id"]
                    example_text = _get_example_text(example)

                    for entity in example["entities"]:

                        for msg in self._check_offsets(
                            example_id=example_id,
                            split=split,
                            example_text=example_text,
                            offsets=entity["offsets"],
                            texts=entity["text"],
                        ):

                            entity_id = entity["id"]
                            errors.append(f"Example:{example_id} - entity:{entity_id} " + msg)

        if len(errors) > 0:
            logger.warning(msg="\n".join(errors) + OFFSET_ERROR_MSG)

    # UNTESTED: no dataset example
    def test_events_offsets(self, dataset_seacrowd: DatasetDict):
        """
        Verify that the events' trigger offsets are correct,
        i.e.: trigger text == text extracted via the trigger offsets
        """
        logger.info("KB ONLY: Checking event offsets")
        errors = []

        for split in dataset_seacrowd:

            if "events" in dataset_seacrowd[split].features:

                for example in dataset_seacrowd[split]:

                    example_id = example["id"]
                    example_text = _get_example_text(example)

                    for event in example["events"]:

                        for msg in self._check_offsets(
                            example_id=example_id,
                            split=split,
                            example_text=example_text,
                            offsets=event["trigger"]["offsets"],
                            texts=event["trigger"]["text"],
                        ):

                            event_id = event["id"]
                            errors.append(f"Example:{example_id} - event:{event_id} " + msg)

        if len(errors) > 0:
            logger.warning(msg="\n".join(errors) + OFFSET_ERROR_MSG)

    def test_coref_ids(self, dataset_seacrowd: DatasetDict):
        """
        Verify that coreferences ids are entities

        from `examples/test_n2c2_2011_coref.py`
        """  # noqa
        logger.info("KB ONLY: Checking coref offsets")
        for split in dataset_seacrowd:

            if "coreferences" in dataset_seacrowd[split].features:

                for example in dataset_seacrowd[split]:
                    example_id = example["id"]
                    entity_lookup = {ent["id"]: ent for ent in example["entities"]}

                    # check all coref entity ids are in entity lookup
                    for coref in example["coreferences"]:
                        for entity_id in coref["entity_ids"]:
                            assert entity_id in entity_lookup, f"Split:{split} - Example:{example_id} - Entity:{entity_id} not found!"

    def test_multiple_choice(self, dataset_seacrowd: DatasetDict):
        """
        Verify that each answer in a multiple choice Q/A task is in choices.
        """  # noqa
        logger.info("QA ONLY: Checking multiple choice")
        for split in dataset_seacrowd:

            for example in dataset_seacrowd[split]:

                if len(example["choices"]) > 0:
                    assert example["type"] == "multiple_choice", f"example has populated choices, but is not type 'multiple_choice' {example}"  # can change this to "in" if we include ranking

                if example["type"] == "multiple_choice":
                    assert len(example["choices"]) > 0, f"example has type 'multiple_choice' but no values in 'choices' {example}"

                    for answer in example["answer"]:
                        assert answer in example["choices"], f"example has an answer that is not present in 'choices' {example}"

    def test_schema(self, schema: str):
        """Search supported tasks within a dataset and verify seacrowd schema"""

        non_empty_features = set()
        if schema == "KB":
            features = kb_features
            for task in self._SUPPORTED_TASKS:
                if task in TASK_TO_FEATURES:
                    non_empty_features.update(TASK_TO_FEATURES[task])
        else:
            features = SCHEMA_TO_FEATURES[schema]

        split_to_feature_counts = self.get_feature_statistics(features=features, schema=schema)
        for split_name, split in self.datasets_seacrowd[schema].items():
            self.assertEqual(len(features), len(split.info.features))
            for key in features:
                if key not in split.info.features:
                    raise AssertionError(f"Required feature '{key}' does not exist")
                self.assertEqual(type(features[key]), type(split.info.features[key]))
            # self.assertEqual(split.info.features, features)
            for non_empty_feature in non_empty_features:
                if split_to_feature_counts[split_name][non_empty_feature] == 0:
                    raise AssertionError(f"Required key '{non_empty_feature}' does not have any instances")

            for feature, count in split_to_feature_counts[split_name].items():
                if count > 0 and feature not in non_empty_features and feature in set().union(*TASK_TO_FEATURES.values()):
                    logger.warning(f"Found instances of '{feature}' but there seems to be no task in 'SUPPORTED_TASKS' for them. Is 'SUPPORTED_TASKS' correct?")

    def _test_is_list(self, msg: str, field: list):
        with self.subTest(
            msg,
            field=field,
        ):
            self.assertIsInstance(field, list)

    def _test_has_only_one_item(self, msg: str, field: list):
        with self.subTest(
            msg,
            field=field,
        ):
            self.assertEqual(len(field), 1)


if __name__ == "__main__":
    logging.basicConfig(level=logging.INFO)

    parser = argparse.ArgumentParser(description="Unit tests for SEACrowd datasets. Args are passed to `datasets.load_dataset`")

    parser.add_argument("path", type=str, help="path to dataloader script (e.g. examples/n2c2_2011.py)")
    parser.add_argument(
        "--schema",
        type=str,
        default=None,
        required=False,
        choices=list(VALID_SCHEMAS),
        help="by default, seacrowd schemas will be discovered from _SUPPORTED_TASKS. use this to explicitly test only one schema.",
    )
    parser.add_argument(
        "--subset_id",
        default=None,
        required=False,
        help="by default, subset_id will be generated from path (e.g. if path=datasets/smsa.py then subset_id=smsa). the config name is then constructed as config_name=<subset_id>_seacrowd_<schema>. use this to explicitly set the subset_id for the config name you want to test.",
    ),
    parser.add_argument("--data_dir", type=str, default=None)
    parser.add_argument("--use_auth_token", default=None)

    args = parser.parse_args()
    logger.info(f"args: {args}")

    if args.subset_id is None:
        subset_id = args.path.split(".py")[0].split("/")[-1]
    else:
        subset_id = args.subset_id

    TestDataLoader.PATH = args.path
    TestDataLoader.SUBSET_ID = subset_id
    TestDataLoader.SCHEMA = args.schema
    TestDataLoader.DATA_DIR = args.data_dir
    TestDataLoader.USE_AUTH_TOKEN = args.use_auth_token

    unittest.TextTestRunner().run(TestDataLoader())



from pathlib import Path
from typing import Dict, List, Tuple

import datasets
import pandas as pd

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks

_CITATION = """
@INPROCEEDINGS{8629181,
    author={Ilmania, Arfinda and Abdurrahman and Cahyawijaya, Samuel and Purwarianti, Ayu},
    booktitle={2018 International Conference on Asian Language Processing (IALP)},
    title={Aspect Detection and Sentiment Classification Using Deep Neural Network for Indonesian Aspect-Based Sentiment Analysis},
    year={2018},
    volume={},
    number={},
    pages={62-67},
    doi={10.1109/IALP.2018.8629181
}
"""

_LANGUAGES = ["ind"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_LOCAL = False

_DATASETNAME = "casa"

_DESCRIPTION = """
CASA: An aspect-based sentiment analysis dataset consisting of around a thousand car reviews collected from multiple Indonesian online automobile platforms (Ilmania et al., 2018).
The dataset covers six aspects of car quality.
We define the task to be a multi-label classification task,
where each label represents a sentiment for a single aspect with three possible values: positive, negative, and neutral.
"""

_HOMEPAGE = "https://github.com/IndoNLP/indonlu"

_LICENSE = "CC-BY-SA 4.0"

_URLS = {
    "train": "https://raw.githubusercontent.com/IndoNLP/indonlu/master/dataset/casa_absa-prosa/train_preprocess.csv",
    "validation": "https://raw.githubusercontent.com/IndoNLP/indonlu/master/dataset/casa_absa-prosa/valid_preprocess.csv",
    "test": "https://raw.githubusercontent.com/IndoNLP/indonlu/master/dataset/casa_absa-prosa/test_preprocess.csv",
}

_SUPPORTED_TASKS = [Tasks.ASPECT_BASED_SENTIMENT_ANALYSIS]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class CASA(datasets.GeneratorBasedBuilder):
    """CASA is an aspect based sentiment analysis dataset"""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="casa_source",
            version=SOURCE_VERSION,
            description="CASA source schema",
            schema="source",
            subset_id="casa",
        ),
        SEACrowdConfig(
            name="casa_seacrowd_text_multi",
            version=SEACROWD_VERSION,
            description="CASA Nusantara schema",
            schema="seacrowd_text_multi",
            subset_id="casa",
        ),
    ]

    DEFAULT_CONFIG_NAME = "casa_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "index": datasets.Value("int64"),
                    "sentence": datasets.Value("string"),
                    "fuel": datasets.Value("string"),
                    "machine": datasets.Value("string"),
                    "others": datasets.Value("string"),
                    "part": datasets.Value("string"),
                    "price": datasets.Value("string"),
                    "service": datasets.Value("string"),
                }
            )

        elif self.config.schema == "seacrowd_text_multi":
            features = schemas.text_multi_features(["positive", "neutral", "negative"])

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        train_csv_path = Path(dl_manager.download_and_extract(_URLS["train"]))
        validation_csv_path = Path(dl_manager.download_and_extract(_URLS["validation"]))
        test_csv_path = Path(dl_manager.download_and_extract(_URLS["test"]))

        data_dir = {
            "train": train_csv_path,
            "validation": validation_csv_path,
            "test": test_csv_path,
        }

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": data_dir["train"],
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": data_dir["test"],
                    "split": "test",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": data_dir["validation"],
                    "split": "dev",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        df = pd.read_csv(filepath, sep=",", header="infer").reset_index()
        if self.config.schema == "source":
            for row in df.itertuples():
                entry = {"index": row.index, "sentence": row.sentence, "fuel": row.fuel, "machine": row.machine, "others": row.others, "part": row.part, "price": row.price, "service": row.service}
                yield row.index, entry

        elif self.config.schema == "seacrowd_text_multi":
            for row in df.itertuples():
                entry = {
                    "id": str(row.index),
                    "text": row.sentence,
                    "labels": [label for label in row[3:]],
                }
                yield row.index, entry



import datasets
import pandas as pd

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@article{wibowo2023copal,
  title={COPAL-ID: Indonesian Language Reasoning with Local Culture and Nuances},
  author={Wibowo, Haryo Akbarianto and Fuadi, Erland Hilman and Nityasya, Made Nindyatama and Prasojo, Radityo Eko and Aji, Alham Fikri},
  journal={arXiv preprint arXiv:2311.01012},
  year={2023}
}
"""
_DATASETNAME = "copal"

_DESCRIPTION = """\
COPAL is a novel Indonesian language common sense reasoning dataset. Unlike the previous Indonesian COPA dataset (XCOPA-ID), COPAL-ID incorporates Indonesian local and cultural nuances,
providing a more natural portrayal of day-to-day causal reasoning within the Indonesian cultural sphere.
Professionally written by natives from scratch, COPAL-ID is more fluent and free from awkward phrases, unlike the translated XCOPA-ID.
Additionally, COPAL-ID is presented in both standard Indonesian and Jakartan Indonesian–a commonly used dialect.
It consists of premise, choice1, choice2, question, and label, similar to the COPA dataset.
"""

_HOMEPAGE = "https://huggingface.co/datasets/haryoaw/COPAL"

_LICENSE = Licenses.CC_BY_SA_4_0.value

_URLS = {"test": "https://huggingface.co/datasets/haryoaw/COPAL/resolve/main/test_copal.csv?download=true", "test_colloquial": "https://huggingface.co/datasets/haryoaw/COPAL/resolve/main/test_copal_colloquial.csv?download=true"}

_SUPPORTED_TASKS = [Tasks.COMMONSENSE_REASONING]

_LOCAL = False
_LANGUAGES = ["ind"]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class COPAL(datasets.GeneratorBasedBuilder):

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description="COPAL test source schema",
            schema="source",
            subset_id="copal",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_colloquial_source",
            version=SOURCE_VERSION,
            description="COPAL test colloquial source schema",
            schema="source",
            subset_id="copal",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_qa",
            version=SEACROWD_VERSION,
            description="COPAL test seacrowd schema",
            schema="seacrowd_qa",
            subset_id="copal",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_colloquial_seacrowd_qa",
            version=SEACROWD_VERSION,
            description="COPAL test colloquial seacrowd schema",
            schema="seacrowd_qa",
            subset_id="copal",
        ),
    ]

    DEFAULT_CONFIG_NAME = "copal_source"

    def _info(self):
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "premise": datasets.Value("string"),
                    "choice1": datasets.Value("string"),
                    "choice2": datasets.Value("string"),
                    "question": datasets.Value("string"),
                    "idx": datasets.Value("int64"),
                    "label": datasets.Value("int64"),
                    "terminology": datasets.Value("int64"),
                    "culture": datasets.Value("int64"),
                    "language": datasets.Value("int64"),
                }
            )
        elif self.config.schema == "seacrowd_qa":
            features = schemas.qa_features
            features["meta"] = {"terminology": datasets.Value("int64"), "culture": datasets.Value("int64"), "language": datasets.Value("int64")}

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        data_dir = dl_manager.download_and_extract(_URLS)
        if "colloquial" in self.config.name:
            data_url = data_dir["test_colloquial"]
        else:
            data_url = data_dir["test"]
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"filepath": data_url},
            ),
        ]

    def _generate_examples(self, filepath):
        df = pd.read_csv(filepath, sep=",", header="infer").reset_index()
        if self.config.schema == "source":
            for row in df.itertuples():
                entry = {
                    "premise": row.premise,
                    "choice1": row.choice1,
                    "choice2": row.choice2,
                    "question": row.question,
                    "idx": row.idx,
                    "label": row.label,
                    "terminology": row.Terminology,
                    "culture": row.Culture,
                    "language": row.Language,
                }
                yield row.index, entry

        elif self.config.schema == "seacrowd_qa":
            for row in df.itertuples():
                entry = {
                    "id": row.idx,
                    "question_id": str(row.idx),
                    "document_id": str(row.idx),
                    "question": row.question,
                    "type": "multiple_choice",
                    "choices": [row.choice1, row.choice2],
                    "context": row.premise,
                    "answer": [row.choice1 if row.label == 0 else row.choice2],
                    "meta": {"terminology": row.Terminology, "culture": row.Culture, "language": row.Language},
                }
                yield row.index, entry
        else:
            raise ValueError(f"Invalid config: {self.config.name}")



import os
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks
from seacrowd.utils import schemas
import pandas as pd

_CITATION = """\
@inproceedings{wongso2021causal,
  title={Causal and masked language modeling of Javanese language using transformer-based architectures},
  author={Wongso, Wilson and Setiawan, David Samuel and Suhartono, Derwin},
  booktitle={2021 International Conference on Advanced Computer Science and Information Systems (ICACSIS)},
  pages={1--7},
  year={2021},
  organization={IEEE}
}
"""

_DATASETNAME = "imdb_jv"

_DESCRIPTION = """\
Javanese Imdb Movie Reviews Dataset is a Javanese version of the IMDb Movie Reviews dataset by translating the original English dataset to Javanese.
"""

_HOMEPAGE = "https://huggingface.co/datasets/w11wo/imdb-javanese"

_LANGUAGES = ["ind"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_LOCAL = False

_LICENSE = "Unknown"

_URLS = {
    _DATASETNAME: "https://huggingface.co/datasets/w11wo/imdb-javanese/resolve/main/javanese_imdb_csv.zip",
}

_SUPPORTED_TASKS = [Tasks.SENTIMENT_ANALYSIS]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"

class IMDbJv(datasets.GeneratorBasedBuilder):
    """Javanese Imdb Movie Reviews Dataset is a Javanese version of the IMDb Movie Reviews dataset by translating the original English dataset to Javanese."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="imdb_jv_source",
            version=datasets.Version(_SOURCE_VERSION),
            description="imdb_jv source schema",
            schema="source",
            subset_id="imdb_jv",
        ),
        SEACrowdConfig(
            name="imdb_jv_seacrowd_text",
            version=datasets.Version(_SEACROWD_VERSION),
            description="imdb_jv Nusantara schema",
            schema="seacrowd_text",
            subset_id="imdb_jv",
        ),
    ]

    DEFAULT_CONFIG_NAME = "imdb_jv_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
               {
                    "id": datasets.Value("string"),
                    "text": datasets.Value("string"),
                    "label": datasets.Value("string")
               }
            )
        elif self.config.schema == "seacrowd_text":
            features = schemas.text_features(['1', '0', '-1'])

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )


    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        data_dir = Path(dl_manager.download_and_extract(_URLS[_DATASETNAME]))

        data_files = {
            "train": "javanese_imdb_train.csv",
            "unsupervised": "javanese_imdb_unsup.csv",
            "test": "javanese_imdb_test.csv",
        }

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,

                gen_kwargs={
                    "filepath": os.path.join(data_dir, data_files["train"]),
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name="unsupervised",
                gen_kwargs={
                    "filepath": os.path.join(data_dir, data_files["unsupervised"]),
                    "split": "unsupervised",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": os.path.join(data_dir, data_files["test"]),
                    "split": "test",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        if self.config.schema == "source":
            data = pd.read_csv(filepath)
            length = len(data['label'])
            for id in range(length):
                ex = {
                    "id": str(id),
                    "text": data['text'][id],
                    "label": data['label'][id],
                }
                yield id, ex

        elif self.config.schema == "seacrowd_text":
            data = pd.read_csv(filepath)
            length = len(data['label'])
            for id in range(length):
                ex = {
                    "id": str(id),
                    "text": data['text'][id],
                    "label": data['label'][id],
                }
                yield id, ex
        else:
            raise ValueError(f"Invalid config: {self.config.name}")



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pathlib import Path
from typing import Dict, List, Tuple

import datasets
import pandas as pd

from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import TASK_TO_SCHEMA, Licenses

_CITATION = """\
@inproceedings{maxwelll-smith-foley-2023-automated,
title = "Automated speech recognition of {I}ndonesian-{E}nglish language lessons on {Y}ou{T}ube using transfer learning",
author = "Maxwell-Smith, Zara and Foley, Ben",
editor = "Serikov, Oleg
        and Voloshina, Ekaterina
        and Postnikova, Anna
        and Klyachko, Elena
        and Vylomova, Ekaterina
        and Shavrina, Tatiana
        and Le Ferrand, Eric
        and Malykh, Valentin
        and Tyers, Francis
        and Arkhangelskiy, Timofey
        and Mikhailov, Vladislav",
    booktitle = "Proceedings of the Second Workshop on NLP Applications to Field Linguistics",
    month = may,
    year = "2023",
    address = "Dubrovnik, Croatia",
    publisher = "Association for Computational Linguistics",
    url = "https://aclanthology.org/2023.fieldmatters-1.1",
    doi = "10.18653/v1/2023.fieldmatters-1.1",
    pages = "1--16",
    abstract = "Experiments to fine-tune large multilingual models with limited data from a specific domain or setting has potential
    to improve automatic speech recognition (ASR) outcomes. This paper reports on the use of the Elpis ASR pipeline to fine-tune two
    pre-trained base models, Wav2Vec2-XLSR-53 and Wav2Vec2-Large-XLSR-Indonesian, with various mixes of data from 3 YouTube channels
    teaching Indonesian with English as the language of instruction. We discuss our results inferring new lesson audio (22-46%
    word error rate) in the context of speeding data collection in diverse and specialised settings. This study is an example of how
    ASR can be used to accelerate natural language research, expanding ethically sourced data in low-resource settings.",
}
"""

_DATASETNAME = "oil"

_DESCRIPTION = """\
The Online Indonesian Learning (OIL) dataset or corpus currently contains lessons from three Indonesian teachers who have posted content on YouTube.
"""

_HOMEPAGE = "https://huggingface.co/datasets/ZMaxwell-Smith/OIL"

_LANGUAGES = ["eng", "ind"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)

_LICENSE = Licenses.CC_BY_NC_ND_4_0.value

_LOCAL = False

_URLS = {
    _DATASETNAME: {"train": "https://huggingface.co/api/datasets/ZMaxwell-Smith/OIL/parquet/default/train/0.parquet"},
}

_SUPPORTED_TASKS = []
_SUPPORTED_SCHEMA_STRINGS = [f"seacrowd_{str(TASK_TO_SCHEMA[task]).lower()}" for task in _SUPPORTED_TASKS]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class OIL(datasets.GeneratorBasedBuilder):
    """The Online Indonesian Learning (OIL) dataset or corpus currently contains lessons from three Indonesian teachers who have posted content on YouTube."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:

        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "audio": datasets.Audio(decode=False),
                    "label": datasets.ClassLabel(num_classes=98),
                }
            )

        else:
            raise ValueError(f"Invalid config: {self.config.name}")

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""

        urls = _URLS[_DATASETNAME]
        train_path = dl_manager.download_and_extract(urls["train"])

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": train_path,
                    "split": "train",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""

        if self.config.schema == "source":

            df = pd.read_parquet(filepath)

            for index, row in df.iterrows():
                yield index, row.to_dict()

        else:
            raise ValueError(f"Invalid config: {self.config.name}")



from pathlib import Path
from typing import Dict, List, Tuple
import datasets
import pandas as pd
from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import (DEFAULT_SEACROWD_VIEW_NAME,
                                       DEFAULT_SOURCE_VIEW_NAME, Tasks)
_LOCAL = False
_DATASETNAME = "nusaparagraph_emot"
_SOURCE_VIEW_NAME = DEFAULT_SOURCE_VIEW_NAME
_UNIFIED_VIEW_NAME = DEFAULT_SEACROWD_VIEW_NAME
_LANGUAGES = [
    "btk", "bew", "bug", "jav", "mad", "mak", "min", "mui", "rej", "sun"
]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_CITATION = """\
@unpublished{anonymous2023nusawrites:,        
    title={NusaWrites: Constructing High-Quality Corpora for Underrepresented and Extremely Low-Resource Languages},        
    author={Anonymous},        
    journal={OpenReview Preprint},        
    year={2023},        
    note={anonymous preprint under review}    
}
"""
_DESCRIPTION = """\
Democratizing access to natural language processing (NLP) technology is crucial, especially for underrepresented and extremely low-resource languages. Previous research has focused on developing labeled and unlabeled corpora for these languages through online scraping and document translation. While these methods have proven effective and cost-efficient, we have identified limitations in the resulting corpora, including a lack of lexical diversity and cultural relevance to local communities. To address this gap, we conduct a case study on Indonesian local languages. We compare the effectiveness of online scraping, human translation, and paragraph writing by native speakers in constructing datasets. Our findings demonstrate that datasets generated through paragraph writing by native speakers exhibit superior quality in terms of lexical diversity and cultural content. In addition, we present the NusaWrites benchmark, encompassing 12 underrepresented and extremely low-resource languages spoken by millions of individuals in Indonesia. Our empirical experiment results using existing multilingual large language models conclude the need to extend these models to more underrepresented languages.
We introduce a novel high quality human curated corpora, i.e., NusaMenulis, which covers 12 languages spoken in Indonesia. The resource extend the coverage of languages to 5 new languages, i.e., Ambon (abs), Bima (bhp), Makassarese (mak), Palembang / Musi (mui), and Rejang (rej).
For the emotion recognition task, we cover the 6 basic emotions (Ekman, 1992): fear, disgusted, sad, happy, angry, and surprise, and an additional emotion label: shame (Poulson and of Tasmania. School of Management, 2000.
"""
_HOMEPAGE = "https://github.com/IndoNLP/nusa-writes"
_LICENSE = "Creative Commons Attribution Share-Alike 4.0 International"
_SUPPORTED_TASKS = [Tasks.EMOTION_CLASSIFICATION]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"
_URLS = {
    "train":
    "https://raw.githubusercontent.com/IndoNLP/nusa-writes/main/data/nusa_alinea-emot-{lang}-train.csv",
    "validation":
    "https://raw.githubusercontent.com/IndoNLP/nusa-writes/main/data/nusa_alinea-emot-{lang}-valid.csv",
    "test":
    "https://raw.githubusercontent.com/IndoNLP/nusa-writes/main/data/nusa_alinea-emot-{lang}-test.csv",
}
def seacrowd_config_constructor(lang, schema, version):
    """Construct SEACrowdConfig with nusaparagraph_emot_{lang}_{schema} as the name format"""
    if schema != "source" and schema != "seacrowd_text":
        raise ValueError(f"Invalid schema: {schema}")
    if lang == "":
        return SEACrowdConfig(
            name="nusaparagraph_emot_{schema}".format(schema=schema),
            version=datasets.Version(version),
            description=
            "nusaparagraph_emot with {schema} schema for all 10 languages".
            format(schema=schema),
            schema=schema,
            subset_id="nusaparagraph_emot",
        )
    else:
        return SEACrowdConfig(
            name="nusaparagraph_emot_{lang}_{schema}".format(lang=lang,
                                                             schema=schema),
            version=datasets.Version(version),
            description=
            "nusaparagraph_emot with {schema} schema for {lang} language".
            format(lang=lang, schema=schema),
            schema=schema,
            subset_id="nusaparagraph_emot",
        )
LANGUAGES_MAP = {
    "btk": "batak",
    "bew": "betawi",
    "bug": "buginese",
    "jav": "javanese",
    "mad": "madurese",
    "mak": "makassarese",
    "min": "minangkabau",
    "mui": "musi",
    "rej": "rejang",
    "sun": "sundanese"
}
class NusaParagraphEmot(datasets.GeneratorBasedBuilder):
    """NusaParagraph-Emot is a 7-labels (fear, disgusted, sad, happy, angry, surprise, and shame) emotion classification dataset for 10 Indonesian local languages."""
    BUILDER_CONFIGS = ([
        seacrowd_config_constructor(lang, "source", _SOURCE_VERSION)
        for lang in LANGUAGES_MAP
    ] + [
        seacrowd_config_constructor(lang, "seacrowd_text",
                                     _SEACROWD_VERSION)
        for lang in LANGUAGES_MAP
    ] + [
        seacrowd_config_constructor("", "source", _SOURCE_VERSION),
        seacrowd_config_constructor("", "seacrowd_text", _SEACROWD_VERSION)
    ])
    DEFAULT_CONFIG_NAME = "nusaparagraph_emot_ind_source"
    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features({
                "id": datasets.Value("string"),
                "text": datasets.Value("string"),
                "label": datasets.Value("string"),
            })
        elif self.config.schema == "seacrowd_text":
            features = schemas.text_features([
                "fear", "disgusted", "sad", "happy", "angry", "surprise",
                "shame"
            ])
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )
    def _split_generators(
            self, dl_manager: datasets.DownloadManager
    ) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        if self.config.name == "nusaparagraph_emot_source" or self.config.name == "nusaparagraph_emot_seacrowd_text":
            # Load all 12 languages
            train_csv_path = dl_manager.download_and_extract([
                _URLS["train"].format(lang=lang)
                for lang in LANGUAGES_MAP
            ])
            validation_csv_path = dl_manager.download_and_extract([
                _URLS["validation"].format(lang=lang)
                for lang in LANGUAGES_MAP
            ])
            test_csv_path = dl_manager.download_and_extract([
                _URLS["test"].format(lang=lang)
                for lang in LANGUAGES_MAP
            ])
        else:
            lang = self.config.name.split('_')[2]
            train_csv_path = Path(
                dl_manager.download_and_extract(
                    _URLS["train"].format(lang=lang)))
            validation_csv_path = Path(
                dl_manager.download_and_extract(
                    _URLS["validation"].format(lang=lang)))
            test_csv_path = Path(
                dl_manager.download_and_extract(
                    _URLS["test"].format(lang=lang)))
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": train_csv_path},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"filepath": validation_csv_path},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"filepath": test_csv_path},
            ),
        ]
    def _generate_examples(self, filepath: Path) -> Tuple[int, Dict]:
        if self.config.schema != "source" and self.config.schema != "seacrowd_text":
            raise ValueError(f"Invalid config: {self.config.name}")
        if self.config.name == "nusaparagraph_emot_source" or self.config.name == "nusaparagraph_emot_seacrowd_text":
            ldf = []
            for fp in filepath:
                ldf.append(pd.read_csv(fp))
            df = pd.concat(ldf, axis=0, ignore_index=True).reset_index()
            # Have to use index instead of id to avoid duplicated key
            df = df.drop(columns=["id"]).rename(columns={"index": "id"})
        else:
            df = pd.read_csv(filepath).reset_index()
        for row in df.itertuples():
            ex = {"id": str(row.id), "text": row.text, "label": row.label}
            yield row.id, ex



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import os
import re
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@article{sornlertlamvanich1999building,
  title={Building a Thai part-of-speech tagged corpus (ORCHID)},
  author={Sornlertlamvanich, Virach and Takahashi, Naoto and Isahara, Hitoshi},
  journal={Journal of the Acoustical Society of Japan (E)},
  volume={20},
  number={3},
  pages={189--198},
  year={1999},
  publisher={Acoustical Society of Japan}
}
"""

_DATASETNAME = "orchid_pos"

_DESCRIPTION = """\
The ORCHID corpus is a Thai part-of-speech (POS) tagged dataset, resulting from a collaboration between\
Japan's Communications Research Laboratory (CRL) and Thailand's National Electronics and Computer Technology\
Center (NECTEC). It is structured at three levels: paragraph, sentence, and word. The dataset incorporates a\
unique tagset designed for use in multi-lingual machine translation projects, and is tailored to address the\
challenges of Thai text, which lacks explicit word and sentence boundaries, punctuation, and inflection.\
This dataset includes text information along with numbering for retrieval, and employs a probabilistic trigram\
model for word segmentation and POS tagging. The ORCHID corpus is specifically structured to reduce ambiguity in\
POS assignments, making it a valuable resource for Thai language processing and computational linguistics research.
"""

_HOMEPAGE = "https://github.com/wannaphong/corpus_mirror/releases/tag/orchid-v1.0"

_LANGUAGES = ["tha"]

_LICENSE = Licenses.CC_BY_NC_SA_3_0.value

_LOCAL = False

_URLS = {
    _DATASETNAME: "https://github.com/wannaphong/corpus_mirror/releases/download/orchid-v1.0/orchid97.crp.utf",
}

_SUPPORTED_TASKS = [Tasks.POS_TAGGING]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class OrchidPOSDataset(datasets.GeneratorBasedBuilder):

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_seq_label",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema="seacrowd_seq_label",
            subset_id=f"{_DATASETNAME}",
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        label_names = [
            "NPRP",
            "NCNM",
            "NONM",
            "NLBL",
            "NCMN",
            "NTTL",
            "PPRS",
            "PDMN",
            "PNTR",
            "PREL",
            "VACT",
            "VSTA",
            "VATT",
            "XVBM",
            "XVAM",
            "XVMM",
            "XVBB",
            "XVAE",
            "DDAN",
            "DDAC",
            "DDBQ",
            "DDAQ",
            "DIAC",
            "DIBQ",
            "DIAQ",
            "DCNM",
            "DONM",
            "ADVN",
            "ADVI",
            "ADVP",
            "ADVS",
            "CNIT",
            "CLTV",
            "CMTR",
            "CFQC",
            "CVBL",
            "JCRG",
            "JCMP",
            "JSBR",
            "RPRE",
            "INT",
            "FIXN",
            "FIXV",
            "EAFF",
            "EITT",
            "NEG",
            "PUNC",
            "CMTR@PUNC",
        ]
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "ttitle": datasets.Value("string"),
                    "etitle": datasets.Value("string"),
                    "tauthor": datasets.Value("string"),
                    "eauthor": datasets.Value("string"),
                    "tinbook": datasets.Value("string"),
                    "einbook": datasets.Value("string"),
                    "tpublisher": datasets.Value("string"),
                    "epublisher": datasets.Value("string"),
                    "year": datasets.Value("string"),
                    "file": datasets.Value("string"),
                    "tokens": datasets.Sequence(datasets.Value("string")),
                    "labels": datasets.Sequence(datasets.ClassLabel(names=label_names)),
                }
            )

        elif self.config.schema == "seacrowd_seq_label":
            features = schemas.seq_label_features(label_names)

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        urls = _URLS[_DATASETNAME]
        data_dir = dl_manager.download_and_extract(urls)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": os.path.join(data_dir, ""),
                    "split": "train",
                },
            )
        ]

    def _get_tokens_labels(self, paragraphs):
        tokens = []
        labels = []
        token_mapping = {
            "<space>": " ",
            "<exclamation>": "!",
            "<quotation>": '"',
            "<number>": "#",
            "<dollar>": "$",
            "<percent>": "%",
            "<ampersand>": "&",
            "<apostrophe>": "'",
            "<slash>": "/",
            "<colon>": ":",
            "<semi_colon>": ";",
            "<less_than>": "<",
            "<equal>": "=",
            "<greater than>": ">",
            "<question_mark>": "?",
            "<at_mark>": "@",
            "<left_parenthesis>": "(",
            "<left_square_bracket>": "[",
            "<right_parenthesis>": ")",
            "<right_square_bracket>": "]",
            "<asterisk>": "*",
            "<circumflex_accent>": "^",
            "<plus>": "+",
            "<low_line>": "_",
            "<comma>": ",",
            "left_curly_bracket": "{",
            "<minus>": "-",
            "<right_curly_bracket>": "}",
            "<full_stop>": ".",
            "<tilde>": "~",
        }
        for paragraph in paragraphs:
            sentences = re.split(r"#\d+\n", paragraph)
            for sentence in sentences[1:]:
                token_pos_pairs = sentence.split("//")[1]
                for token_pos_pair in token_pos_pairs.split("\n")[1:-1]:
                    if "/" in token_pos_pair:
                        token = token_pos_pair.split("/")[0]
                        tokens.append(token_mapping[token] if token in token_mapping.keys() else token)
                        labels.append(token_pos_pair.split("/")[1])
                    else:
                        token = token_pos_pair.split("@")[0]
                        tokens.append(token_mapping[token] if token in token_mapping.keys() else token)
                        labels.append(token_pos_pair.split("@")[1])
        return tokens, labels

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        file_content = open(filepath, "r").read()
        texts = file_content.split("%TTitle:")

        idx = 0
        for text in texts[1:]:
            file_part = text.split("%File")[-1]
            tokens, labels = self._get_tokens_labels(re.split(r"#P\d+\n", file_part)[1:])
            if self.config.schema == "source":
                parts = text.split("%")
                example = {
                    "ttitle": parts[0],
                    "etitle": ":".join(parts[1].split(":")[1:]).strip(),
                    "tauthor": ":".join(parts[2].split(":")[1:]).strip(),
                    "eauthor": ":".join(parts[3].split(":")[1:]).strip(),
                    "tinbook": ":".join(parts[4].split(":")[1:]).strip(),
                    "einbook": ":".join(parts[5].split(":")[1:]).strip(),
                    "tpublisher": ":".join(parts[6].split(":")[1:]).strip(),
                    "epublisher": ":".join(parts[7].split(":")[1:]).strip(),
                    "year": ":".join(parts[9].split(":")[1:]).strip(),
                    "file": file_part.strip(),
                    "tokens": tokens,
                    "labels": labels,
                }
            elif self.config.schema == "seacrowd_seq_label":
                example = {
                    "id": idx,
                    "tokens": tokens,
                    "labels": labels,
                }
            yield idx, example
            idx += 1



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pathlib import Path
from typing import Dict, List, Tuple

import datasets
import pandas as pd

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import TASK_TO_SCHEMA, Licenses, Tasks

_CITATION = """\
@misc{mtet,
  doi = {10.48550/ARXIV.2210.05610},
  url = {https://arxiv.org/abs/2210.05610},
  author = {Ngo, Chinh and Trinh, Trieu H. and Phan, Long and Tran, Hieu and Dang, Tai and Nguyen, Hieu and Nguyen, Minh and Luong, Minh-Thang},
  keywords = {Computation and Language (cs.CL), Artificial Intelligence (cs.AI), FOS: Computer and information sciences, FOS: Computer and information sciences},
  title = {MTet: Multi-domain Translation for English and Vietnamese},
  publisher = {arXiv},
  year = {2022},
  copyright = {Creative Commons Attribution 4.0 International}
}
"""

_DATASETNAME = "vi_pubmed"

_DESCRIPTION = """\
20M Vietnamese PubMed biomedical abstracts translated by the state-of-the-art English-Vietnamese Translation project. The data has been used as unlabeled dataset for pretraining a Vietnamese Biomedical-domain Transformer model.
"""

_HOMEPAGE = "https://huggingface.co/datasets/VietAI/vi_pubmed"

_LANGUAGES = ["eng", "vie"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)

_LICENSE = Licenses.OTHERS.value

_LOCAL = False

_URLS = {
    _DATASETNAME: {
        "pubmed22": [
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/0.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/1.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/2.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/3.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/4.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/5.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/6.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/7.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/8.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/9.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/10.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/11.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/12.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/13.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/14.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/15.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/16.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/17.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/18.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/19.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/20.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/21.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/22.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/23.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/24.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/25.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/26.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/27.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/28.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/29.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/30.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/31.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/32.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/33.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/34.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/35.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/36.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/37.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/38.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/39.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/40.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/41.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/42.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/43.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/44.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/45.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/46.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/47.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/48.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/49.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/50.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/51.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/52.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/53.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/54.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/55.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/56.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/57.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/58.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/59.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/60.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/61.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/62.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/63.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/64.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/65.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/66.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/67.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/68.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/69.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/70.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/71.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/72.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/73.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/74.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/75.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/76.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/77.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/78.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/79.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/80.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/81.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/82.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/83.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/84.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/85.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/86.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/87.parquet",
            "https://huggingface.co/api/datasets/VietAI/vi_pubmed/parquet/default/pubmed22/88.parquet",
        ]
    },
}

_SUPPORTED_TASKS = [Tasks.MACHINE_TRANSLATION]
_SUPPORTED_SCHEMA_STRINGS = [f"seacrowd_{str(TASK_TO_SCHEMA[task]).lower()}" for task in _SUPPORTED_TASKS]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class ViPubmed(datasets.GeneratorBasedBuilder):
    """20M Vietnamese PubMed biomedical abstracts translated by the state-of-the-art English-Vietnamese Translation project. The data has been used as unlabeled dataset for pretraining a Vietnamese Biomedical-domain Transformer model."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
    ]

    seacrowd_schema_config: list[SEACrowdConfig] = []

    for seacrowd_schema in _SUPPORTED_SCHEMA_STRINGS:

        seacrowd_schema_config.append(
            SEACrowdConfig(
                name=f"{_DATASETNAME}_{seacrowd_schema}",
                version=SEACROWD_VERSION,
                description=f"{_DATASETNAME} {seacrowd_schema} schema",
                schema=f"{seacrowd_schema}",
                subset_id=f"{_DATASETNAME}",
            )
        )

    BUILDER_CONFIGS.extend(seacrowd_schema_config)

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:

        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "en": datasets.Value("string"),
                    "vi": datasets.Value("string"),
                }
            )

        elif self.config.schema == f"seacrowd_{str(TASK_TO_SCHEMA[Tasks.MACHINE_TRANSLATION]).lower()}":
            features = schemas.text2text_features

        else:
            raise ValueError(f"Invalid config: {self.config.name}")

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""

        split_name = "pubmed22"
        paths = dl_manager.download_and_extract(_URLS[_DATASETNAME][split_name])

        return [
            datasets.SplitGenerator(
                name=split_name,
                gen_kwargs={
                    "paths": paths,
                    "split": split_name,
                },
            ),
        ]

    def _generate_examples(self, paths: list[Path], split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""

        idx = 0

        if self.config.schema == "source":

            for path in paths:
                df = pd.read_parquet(path)

                for _, row in df.iterrows():
                    yield idx, row.to_dict()
                    idx += 1

        elif self.config.schema == f"seacrowd_{str(TASK_TO_SCHEMA[Tasks.MACHINE_TRANSLATION]).lower()}":
            for path in paths:
                df = pd.read_parquet(path)

                df["id"] = df.index + idx
                df.rename(columns={"en": "text_1"}, inplace=True)
                df.rename(columns={"vi": "text_2"}, inplace=True)
                df = df.assign(text_1_name="en").astype({"text_1_name": "str"})
                df = df.assign(text_2_name="vi").astype({"text_2_name": "str"})

                for _, row in df.iterrows():
                    yield idx, row.to_dict()
                    idx += 1

        else:
            raise ValueError(f"Invalid config: {self.config.name}")
